@@ -43,7 +43,10 @@
 #include "client_list.h"
 #include "util.h"
 #include "http_microhttpd_utils.h"
+#include "http_microhttpd.h"
 
+#define ENABLE 1
+#define DISABLE 0
 
 extern pthread_mutex_t client_list_mutex;
 extern pthread_mutex_t config_mutex;
@@ -55,7 +58,7 @@ unsigned int authenticated_since_start = 0;
 static void binauth_action(t_client *client, const char *reason, char *customdata)
 {
 	s_config *config = config_get_config();
-	char lockfile[] = "/tmp/ndsctl.lock";
+	char *lockfile;
 	FILE *fd;
 	time_t now = time(NULL);
 	int seconds = 60 * config->session_timeout;
@@ -71,11 +74,16 @@ static void binauth_action(t_client *client, const char *reason, char *customdat
 	}
 
 	if (config->binauth) {
+		// ndsctl will deadlock if run within the BinAuth script so lock it
+		safe_asprintf(&lockfile, "%s/ndsctl.lock", config->tmpfsmountpoint);
+
+		if ((fd = fopen(lockfile, "r")) == NULL) {
+			//No lockfile, so create one
+			fd = fopen(lockfile, "w");
+		}
+
 		uh_urlencode(customdata_enc, sizeof(customdata_enc), customdata, strlen(customdata));
 		debug(LOG_DEBUG, "binauth_action: customdata_enc [%s]", customdata_enc);
-		// ndsctl will deadlock if run within the BinAuth script so we must lock it
-		//Create lock
-		fd = fopen(lockfile, "w");
 
 		// get client's current session start and end
 		sessionstart = client->session_start;
@@ -97,7 +105,7 @@ static void binauth_action(t_client *client, const char *reason, char *customdat
 			sessionstart = now;
 		}
 
-		debug(LOG_NOTICE, "BinAuth %s - client session end time: [ %lu ]", reason, sessionend);
+		debug(LOG_DEBUG, "BinAuth %s - client session end time: [ %lu ]", reason, sessionend);
 
 		execute("%s %s %s %llu %llu %lu %lu %s %s",
 			config->binauth,
@@ -112,8 +120,11 @@ static void binauth_action(t_client *client, const char *reason, char *customdat
 		);
 
 		// unlock ndsctl
-		fclose(fd);
-		remove(lockfile);
+		if (fd) {
+			fclose(fd);
+			remove(lockfile);
+		}
+		free(lockfile);
 	}
 }
 
@@ -147,7 +158,7 @@ static int auth_change_state(t_client *client, const unsigned int new_state, con
 				client->download_quota = config->download_quota;
 			}
 
-			debug(LOG_INFO, "auth_change_state > authenticated - download_rate [%llu] upload_rate [%llu] ",
+			debug(LOG_DEBUG, "auth_change_state > authenticated - download_rate [%llu] upload_rate [%llu] ",
 				client->download_rate,
 				client->upload_rate
 			);
@@ -222,6 +233,7 @@ fw_refresh_client_list(void)
 	unsigned long long int download_bytes, upload_bytes;
 	unsigned long long int uprate;
 	unsigned long long int downrate;
+	int action;
 
 	debug(LOG_DEBUG, "Rate Check Window is set to %u period(s) of checkinterval", config->rate_check_window);
 
@@ -326,6 +338,7 @@ fw_refresh_client_list(void)
 		// Now we need to process rate quotas, so first refresh the connection state in case it has changed
 		conn_state = cp1->fw_connection_state;
 
+
 		if (conn_state != FW_MARK_PREAUTHENTICATED) {
 
 			debug(LOG_DEBUG, "Window start [%lu] - window counter [%u]",
@@ -340,12 +353,13 @@ fw_refresh_client_list(void)
 
 			durationsecs = (now - cp1->window_start);
 
-			if (durationsecs <= (config->checkinterval * config->rate_check_window)) {
+			if (cp1->window_counter > 1) {
 				--cp1->window_counter;
+				// skip to next client
 				continue;
 			}
 
-			if (cp1->initial_loop == 0) {
+			if (cp1->window_counter == 1) {
 				download_bytes = (cp1->counters.incoming - cp1->counters.in_window_start);
 				upload_bytes = (cp1->counters.outgoing - cp1->counters.out_window_start);
 				downrate = (download_bytes / 125 / durationsecs); // kbits/sec
@@ -365,50 +379,109 @@ fw_refresh_client_list(void)
 					cp1->upload_rate, uprate
 				);
 
-				if (cp1->download_rate > 0 && cp1->download_rate <= downrate && cp1->rate_exceeded == 0) {
-					//download rate has exceeded quota so deauthenticate the client
 
-					debug(LOG_NOTICE, "Download RATE quota reached for: %s %s, in: %llukbits/s, out: %llukbits/s",
-						cp1->ip, cp1->mac,
-						downrate,
-						uprate
-					);
 
-					cp1->rate_exceeded = 1;
-					iptables_do_command("-I FORWARD -s %s -j DROP", cp1->ip);
-				} else if (cp1->upload_rate > 0 && cp1->upload_rate <= uprate && cp1->rate_exceeded == 0) {
-					//upload rate has exceeded quota so deauthenticate the client
+				debug(LOG_DEBUG, "rate_exceeded flag is: %u", cp1->rate_exceeded);
 
-					debug(LOG_NOTICE, "Upload RATE quota reached for: %s %s, in: %llukbits/s, out: %llukbits/s",
-						cp1->ip, cp1->mac,
-						downrate,
-						uprate
-					);
+				//Handle download rate limiting
+				debug(LOG_DEBUG, "cp1->download_rate: %llu downrate: %llu", cp1->download_rate, downrate);
 
-					cp1->rate_exceeded = 1;
-					iptables_do_command("-I FORWARD -s %s -j DROP", cp1->ip);
+				/*	OR with set bit sets the bit in the variable
+					AND with set bit checks if bit is set in variable
+					XOR with set bit toggles bit in variable
+					In variable rate_exceeded we are interested in bits zero and one
+					Bit zero pertains to download
+					Bit one pertains to upload
+					So variable can have integer values of 0, 1, 2 or 3
+				*/
+
+				if ((cp1->rate_exceeded&1) == 0) {
+					// note checked for bit 0 of rate_exceeded set to 0, it was so we are here 
+					if (cp1->download_rate > 0 && cp1->download_rate <= downrate) {
+
+						debug(LOG_INFO, "Download RATE quota reached for: %s %s, in: %llukbits/s, out: %llukbits/s",
+							cp1->ip, cp1->mac,
+							downrate,
+							uprate
+						);
+						action = ENABLE;
+						iptables_download_ratelimit_enable(cp1, action);
+						//bit 0 is not set so toggle it to signify rate limiting is on
+						cp1->rate_exceeded = cp1->rate_exceeded^1;
+					}
 				}
 
-				if (cp1->download_rate >= downrate && cp1->upload_rate >= uprate && cp1->rate_exceeded == 1) {
-					cp1->rate_exceeded = 0;
-					iptables_do_command("-D FORWARD -s %s -j DROP", cp1->ip);
+				if ((cp1->rate_exceeded&1) == 1) {
+					// note checked for bit 0 of rate_exceeded set to 1, it was so we are here 
+					if (cp1->download_rate > 0 && cp1->download_rate >= downrate) {
+						debug(LOG_INFO, "Download RATE below quota threshold - bursting allowed: %s %s, in: %llukbits/s, out: %llukbits/s",
+							cp1->ip, cp1->mac,
+							downrate,
+							uprate
+						);
+
+						action = DISABLE;
+						iptables_download_ratelimit_enable(cp1, action);
+						//bit 0 is set so toggle it to signify rate limiting is off
+						cp1->rate_exceeded = cp1->rate_exceeded^1;
+					}
 				}
 
-				if (cp1->window_counter == 0) { // Start new window
-					cp1->window_start = now;
-					cp1->window_counter = config->rate_check_window;
-					cp1->counters.in_window_start = cp1->counters.incoming;
-					cp1->counters.out_window_start = cp1->counters.outgoing;
+
+
+
+				//Handle upload rate limiting
+				debug(LOG_DEBUG, "cp1->upload_rate: %llu uprate: %llu", cp1->upload_rate, uprate);
+
+				/*	OR with set bit sets the bit in the variable
+					AND with set bit checks if bit is set in variable
+					XOR with set bit toggles bit in variable
+					In variable rate_exceeded we are interested in bits zero and one
+					Bit zero pertains to download
+					Bit one pertains to upload
+					So variable can have integer values of 0, 1, 2 or 3
+				*/
+				if ((cp1->rate_exceeded&2) == 0) {
+					// note checked for bit 1 of rate_exceeded set to 0, it was so we are here 
+					if (cp1->upload_rate > 0 && cp1->upload_rate <= uprate) {
+
+						debug(LOG_INFO, "Upload RATE quota reached for: %s %s, in: %llukbits/s, out: %llukbits/s",
+							cp1->ip, cp1->mac,
+							downrate,
+							uprate
+						);
+
+						action = ENABLE;
+						iptables_upload_ratelimit_enable(cp1, action);
+						//bit 1 is not set so toggle it to signify rate limiting is on
+						cp1->rate_exceeded = cp1->rate_exceeded^2;
+					}
 				}
-			} else {
-				//reset initial loop and start new window
-				cp1->initial_loop = 0;
+
+				if ((cp1->rate_exceeded&2) == 2) {
+					// note checked for bit 1 of rate_exceeded set to 1, it was so we are here 
+					if (cp1->upload_rate > 0 && cp1->upload_rate > uprate) {
+
+						debug(LOG_INFO, "Upload RATE below quota threshold - bursting allowed: %s %s, in: %llukbits/s, out: %llukbits/s",
+							cp1->ip, cp1->mac,
+							downrate,
+							uprate
+						);
+
+						action = DISABLE;
+						iptables_upload_ratelimit_enable(cp1, action);
+						//bit 1 is set so toggle it to signify rate limiting is off
+						cp1->rate_exceeded = cp1->rate_exceeded^2;
+					}
+				}
+
+
+				// start new window
 				cp1->window_start = now;
 				cp1->window_counter = config->rate_check_window;
 				cp1->counters.in_window_start = cp1->counters.incoming;
 				cp1->counters.out_window_start = cp1->counters.outgoing;
 			}
-
 		}
 	}
 	UNLOCK_CLIENT_LIST();
@@ -424,12 +497,41 @@ thread_client_timeout_check(void *arg)
 	pthread_cond_t cond = PTHREAD_COND_INITIALIZER;
 	pthread_mutex_t cond_mutex = PTHREAD_MUTEX_INITIALIZER;
 	struct timespec timeout;
+	char msg[8] = {0};
+	const char mhd_fail[] = "2";
+	char *testcmd;
+	s_config *config = config_get_config();
+
+	// Build command to check MHD
+	safe_asprintf(&testcmd,
+		"/usr/lib/opennds/libopennds.sh mhdcheck \"%s\"",
+		config->gw_address
+	);
+
 
 	while (1) {
-		debug(LOG_DEBUG, "Running fw_refresh_client_list()");
+
+		// check MHD
+		if (execute_ret_url_encoded(msg, sizeof(msg) - 1, testcmd) == 0) {
+			debug(LOG_DEBUG, "MHD Test Result: %s", msg);
+		} else {
+			debug(LOG_DEBUG, "MHD Test failed: Continuing...");
+		}
+
+		if (strcmp(msg, mhd_fail) == 0) {
+			debug(LOG_INFO, "MHD Watchdog - Restart requested");
+			start_mhd();
+			debug(LOG_INFO, "MHD Restarted");
+		}
+
+		memset(msg, 0, sizeof(msg));
+
+		debug(LOG_DEBUG, "Starting Refresh Client List");
 
 		fw_refresh_client_list();
 
+		debug(LOG_DEBUG, "Client List Refresh is Done");
+
 		// Sleep for config.checkinterval seconds...
 		timeout.tv_sec = time(NULL) + config_get_config()->checkinterval;
 		timeout.tv_nsec = 0;
@@ -444,6 +546,7 @@ thread_client_timeout_check(void *arg)
 		pthread_mutex_unlock(&cond_mutex);
 	}
 
+	free(testcmd);
 	return NULL;
 }
 
@@ -544,22 +647,6 @@ auth_client_untrust(const char *mac)
 	}
 
 	UNLOCK_CONFIG();
-
-/* 
-	if (rc == 0) {
-		LOCK_CLIENT_LIST();
-		t_client * client = client_list_find_by_mac(mac);
-		if (client) {
-			rc = auth_change_state(client, FW_MARK_PREAUTHENTICATED, "manual_untrust", NULL);
-			if (rc == 0) {
-				client->session_start = 0;
-				client->session_end = 0;
-			}
-		}
-		UNLOCK_CLIENT_LIST();
-	}
-*/
-
 	return rc;
 }
 
