@@ -44,7 +44,7 @@
 #include "conf.h"
 #include "auth.h"
 #include "util.h"
-
+#include "http_microhttpd_utils.h"
 
 /** @internal
  * Holds the current configuration of the gateway */
@@ -75,6 +75,7 @@ typedef enum {
 	// TODO: deprecate oGatewayAddress option
 	oGatewayAddress,
 	oGatewayPort,
+	oGatewayFQDN,
 	oFasPort,
 	oFasKey,
 	oFasPath,
@@ -83,20 +84,17 @@ typedef enum {
 	oFasURL,
 	oFasSSL,
 	oLoginOptionEnabled,
+	oThemeSpecPath,
 	oUseOutdatedMHD,
 	oUnescapeCallbackEnabled,
 	oFasSecureEnabled,
 	oHTTPDMaxConn,
 	oWebRoot,
-	oSplashPage,
-	oStatusPage,
-	oRedirectURL,
 	oPreauthIdleTimeout,
 	oAuthIdleTimeout,
 	oCheckInterval,
 	oSetMSS,
 	oMSSValue,
-	oTrafficControl,
 	oRateCheckWindow,
 	oDownloadRate,
 	oUploadRate,
@@ -116,7 +114,13 @@ typedef enum {
 	oFWMarkTrusted,
 	oFWMarkBlocked,
 	oBinAuth,
-	oPreAuth
+	oPreAuth,
+	oWalledGardenFqdnList,
+	oWalledGardenPortList,
+	oFasCustomParametersList,
+	oFasCustomVariablesList,
+	oFasCustomImagesList,
+	oFasCustomFilesList
 } OpCodes;
 
 /** @internal
@@ -137,6 +141,7 @@ static const struct {
 	// TODO: remove/deprecate gatewayaddress keyword
 	{ "gatewayaddress", oGatewayAddress },
 	{ "gatewayport", oGatewayPort },
+	{ "gatewayfqdn", oGatewayFQDN },
 	{ "fasport", oFasPort },
 	{ "faskey", oFasKey },
 	{ "fasremoteip", oFasRemoteIP },
@@ -144,20 +149,17 @@ static const struct {
 	{ "fasurl", oFasURL },
 	{ "fasssl", oFasSSL },
 	{ "login_option_enabled", oLoginOptionEnabled },
+	{ "themespec_path", oThemeSpecPath },
 	{ "use_outdated_mhd", oUseOutdatedMHD },
 	{ "unescape_callback_enabled", oUnescapeCallbackEnabled },
 	{ "fas_secure_enabled", oFasSecureEnabled },
 	{ "faspath", oFasPath },
 	{ "webroot", oWebRoot },
-	{ "splashpage", oSplashPage },
-	{ "statuspage", oStatusPage },
-	{ "redirectURL", oRedirectURL },
 	{ "preauthidletimeout", oPreauthIdleTimeout },
 	{ "authidletimeout", oAuthIdleTimeout },
 	{ "checkinterval", oCheckInterval },
 	{ "setmss", oSetMSS },
 	{ "mssvalue", oMSSValue },
-	{ "trafficcontrol", oTrafficControl },
 	{ "ratecheckwindow", oRateCheckWindow },
 	{ "downloadrate", oDownloadRate },
 	{ "uploadrate", oUploadRate },
@@ -178,6 +180,12 @@ static const struct {
 	{ "fw_mark_blocked", oFWMarkBlocked },
 	{ "binauth", oBinAuth },
 	{ "preauth", oPreAuth },
+	{ "walledgarden_fqdn_list", oWalledGardenFqdnList },
+	{ "walledgarden_port_list", oWalledGardenPortList },
+	{ "fas_custom_parameters_list", oFasCustomParametersList },
+	{ "fas_custom_variables_list", oFasCustomVariablesList },
+	{ "fas_custom_images_list", oFasCustomImagesList },
+	{ "fas_custom_files_list", oFasCustomFilesList },
 	{ NULL, oBadOption },
 };
 
@@ -217,14 +225,16 @@ config_init(void)
 	config.gw_name = safe_strdup(DEFAULT_GATEWAYNAME);
 	config.http_encoded_gw_name = NULL;
 	config.url_encoded_gw_name = NULL;
-	config.gw_interface = NULL;
+	config.gw_fqdn = safe_strdup(DEFAULT_GATEWAYFQDN);
+	config.gw_interface = safe_strdup(DEFAULT_GATEWAYINTERFACE);;
 	config.gw_iprange = safe_strdup(DEFAULT_GATEWAY_IPRANGE);
 	config.gw_address = NULL;
 	config.gw_ip = NULL;
 	config.gw_port = DEFAULT_GATEWAYPORT;
 	config.fas_port = DEFAULT_FASPORT;
-	config.fas_key = NULL;
+	config.fas_key = safe_strdup(DEFAULT_FASKEY);
 	config.login_option_enabled = DEFAULT_LOGIN_OPTION_ENABLED;
+	config.themespec_path = NULL;
 	config.use_outdated_mhd = DEFAULT_USE_OUTDATED_MHD;
 	config.unescape_callback_enabled = DEFAULT_UNESCAPE_CALLBACK_ENABLED;
 	config.fas_secure_enabled = DEFAULT_FAS_SECURE_ENABLED;
@@ -233,21 +243,22 @@ config_init(void)
 	config.fas_url = NULL;
 	config.fas_ssl = NULL;
 	config.fas_hid = NULL;
+	config.custom_params = NULL;
+	config.custom_vars = NULL;
+	config.custom_images = NULL;
+	config.custom_files = NULL;
+	config.tmpfsmountpoint = NULL;
 	config.fas_path = DEFAULT_FASPATH;
 	config.webroot = safe_strdup(DEFAULT_WEBROOT);
-	config.splashpage = safe_strdup(DEFAULT_SPLASHPAGE);
-	config.statuspage = safe_strdup(DEFAULT_STATUSPAGE);
 	config.authdir = safe_strdup(DEFAULT_AUTHDIR);
 	config.denydir = safe_strdup(DEFAULT_DENYDIR);
 	config.preauthdir = safe_strdup(DEFAULT_PREAUTHDIR);
-	config.redirectURL = NULL;
 	config.preauth_idle_timeout = DEFAULT_PREAUTH_IDLE_TIMEOUT,
 	config.auth_idle_timeout = DEFAULT_AUTH_IDLE_TIMEOUT,
 	config.checkinterval = DEFAULT_CHECKINTERVAL;
 	config.daemon = -1;
 	config.set_mss = DEFAULT_SET_MSS;
 	config.mss_value = DEFAULT_MSS_VALUE;
-	config.traffic_control = DEFAULT_TRAFFIC_CONTROL;
 	config.rate_check_window = DEFAULT_RATE_CHECK_WINDOW;
 	config.upload_rate =  DEFAULT_UPLOAD_RATE;
 	config.download_rate = DEFAULT_DOWNLOAD_RATE;
@@ -257,7 +268,6 @@ config_init(void)
 	config.syslog_facility = DEFAULT_SYSLOG_FACILITY;
 	config.log_syslog = DEFAULT_LOG_SYSLOG;
 	config.ndsctl_sock = safe_strdup(DEFAULT_NDSCTL_SOCK);
-	config.internal_sock = safe_strdup(DEFAULT_INTERNAL_SOCK);
 	config.rulesets = NULL;
 	config.trustedmaclist = NULL;
 	config.blockedmaclist = NULL;
@@ -269,6 +279,9 @@ config_init(void)
 	config.ip6 = DEFAULT_IP6;
 	config.binauth = NULL;
 	config.preauth = NULL;
+	config.walledgarden_fqdn_list = NULL;
+	config.walledgarden_port_list = NULL;
+	config.fas_custom_parameters_list = NULL;
 
 	// Set up default FirewallRuleSets, and their empty ruleset policies
 	rs = add_ruleset("trusted-users");
@@ -692,7 +705,22 @@ config_read(const char *filename)
 	char line[MAX_BUF], *s, *p1, *p2;
 	int linenum = 0, opcode, value;
 	struct stat sb;
-	char lockfile[] = "/tmp/ndsctl.lock";
+	char msg[128] = {0};
+	char *cmd = NULL;
+	char *lockfile;
+
+	safe_asprintf(&cmd, "/usr/lib/opennds/libopennds.sh clean");
+
+	execute_ret_url_encoded(msg, sizeof(msg) - 1, cmd);
+	free(cmd);
+
+	if (strlen(msg) == 0) {
+		config.tmpfsmountpoint = safe_strdup("/tmp");
+	} else {
+		config.tmpfsmountpoint = safe_strdup(msg);
+	}
+
+	safe_asprintf(&lockfile, "%s/ndsctl.lock", config.tmpfsmountpoint);
 
 	//Remove ndsctl lock file if it exists
 	if ((fd = fopen(lockfile, "r")) != NULL) {
@@ -700,6 +728,8 @@ config_read(const char *filename)
 		remove(lockfile);
 	}
 
+	free (lockfile);
+
 	debug(LOG_INFO, "Reading configuration file '%s'", filename);
 
 	if (!(fd = fopen(filename, "r"))) {
@@ -773,6 +803,9 @@ config_read(const char *filename)
 		case oGatewayName:
 			config.gw_name = safe_strdup(p1);
 			break;
+		case oGatewayFQDN:
+			config.gw_fqdn = safe_strdup(p1);
+			break;
 		case oGatewayInterface:
 			config.gw_interface = safe_strdup(p1);
 			break;
@@ -805,6 +838,15 @@ config_read(const char *filename)
 				exit(1);
 			}
 			break;
+		case oThemeSpecPath:
+			config.themespec_path = safe_strdup(p1);
+			if (!((stat(p1, &sb) == 0) && S_ISREG(sb.st_mode) && (sb.st_mode & S_IXUSR))) {
+				debug(LOG_ERR, "ThemeSpec program does not exist or is not executeable: %s", p1);
+				debug(LOG_ERR, "Exiting...");
+				exit(1);
+			}
+			debug(LOG_INFO, "Added ThemeSpec [%s]", p1);
+			break;
 		case oUseOutdatedMHD:
 			if (sscanf(p1, "%d", &config.use_outdated_mhd) < 1) {
 				debug(LOG_ERR, "Bad arg %s to option %s on line %d in %s", p1, s, linenum, filename);
@@ -869,6 +911,24 @@ config_read(const char *filename)
 		case oAllowedMACList:
 			parse_allowed_mac_list(p1);
 			break;
+		case oWalledGardenFqdnList:
+			parse_walledgarden_fqdn_list(p1);
+			break;
+		case oWalledGardenPortList:
+			parse_walledgarden_port_list(p1);
+			break;
+		case oFasCustomParametersList:
+			parse_fas_custom_parameters_list(p1);
+			break;
+		case oFasCustomVariablesList:
+			parse_fas_custom_variables_list(p1);
+			break;
+		case oFasCustomImagesList:
+			parse_fas_custom_images_list(p1);
+			break;
+		case oFasCustomFilesList:
+			parse_fas_custom_files_list(p1);
+			break;
 		case oMACmechanism:
 			if (!strcasecmp("allow", p1)) {
 				config.macmechanism = MAC_ALLOW;
@@ -885,18 +945,6 @@ config_read(const char *filename)
 			while((p2 = strrchr(p1,'/')) == (p1 + strlen(p1) - 1)) *p2 = '\0';
 			config.webroot = safe_strdup(p1);
 			break;
-		case oSplashPage:
-			config.splashpage = safe_strdup(p1);
-			break;
-		case oStatusPage:
-			config.statuspage = safe_strdup(p1);
-			break;
-
-		// TODO: Deprecate RedirectURL
-		case oRedirectURL:
-			config.redirectURL = safe_strdup(p1);
-			debug(LOG_WARNING, "RedirectURL is now deprecated, please use FAS to provide this functionality");
-			break;
 		case oAuthIdleTimeout:
 			if (sscanf(p1, "%d", &config.auth_idle_timeout) < 1 || config.auth_idle_timeout < 0) {
 				debug(LOG_ERR, "Bad arg %s to option %s on line %d in %s", p1, s, linenum, filename);
@@ -931,15 +979,6 @@ config_read(const char *filename)
 				exit(1);
 			}
 			break;
-		case oTrafficControl:
-			if ((value = parse_boolean(p1)) != -1) {
-				config.traffic_control = value;
-			} else {
-				debug(LOG_ERR, "Bad arg %s to option %s on line %d in %s", p1, s, linenum, filename);
-				debug(LOG_ERR, "Exiting...");
-				exit(1);
-			}
-			break;
 		case oRateCheckWindow:
 			if (sscanf(p1, "%d", &config.rate_check_window) < 1 || config.rate_check_window < 0) {
 				debug(LOG_ERR, "Bad arg %s to option %s on line %d in %s", p1, s, linenum, filename);
@@ -1072,6 +1111,117 @@ int check_mac_format(const char possiblemac[])
 	return ether_aton(possiblemac) != NULL;
 }
 
+int add_to_walledgarden_fqdn_list(const char possiblefqdn[])
+{
+	char fqdn[128];
+	t_WGFQDN *p = NULL;
+	char possiblefqdn_urlencoded[256] = {0};
+
+	// Sanitise FQDN
+	uh_urlencode(possiblefqdn_urlencoded, sizeof(possiblefqdn_urlencoded), possiblefqdn, strlen(possiblefqdn));
+	debug(LOG_DEBUG, "[%s] is the urlencoded FQDN", possiblefqdn_urlencoded);
+
+	if (strcmp(possiblefqdn_urlencoded, possiblefqdn) == 0) {
+
+		sscanf(possiblefqdn, "%s", fqdn);
+
+		// Add FQDN to head of list
+		p = safe_malloc(sizeof(t_WGFQDN));
+		p->wgfqdn = safe_strdup(fqdn);
+		p->next = config.walledgarden_fqdn_list;
+		config.walledgarden_fqdn_list = p;
+		debug(LOG_INFO, "Added Walled Garden FQDN [%s] to list", possiblefqdn);
+	} else {
+		debug(LOG_WARNING, "Invalid FQDN [%s], please remove from the list; skipping..", possiblefqdn);
+	}
+	return 0;
+}
+
+int add_to_walledgarden_port_list(const char possibleport[])
+{
+	unsigned short int port;
+	t_WGP *p = NULL;
+
+	sscanf(possibleport, "%hu", &port);
+
+	// Add walled garden port to head of list
+	p = safe_malloc(sizeof(t_WGP));
+	p->wgport = port;
+	p->next = config.walledgarden_port_list;
+	config.walledgarden_port_list = p;
+	debug(LOG_INFO, "Added Walled Garden port [%hu] to list", port);
+	return 0;
+}
+
+int add_to_fas_custom_parameters_list(const char possibleparam[])
+{
+	char param[512];
+	t_FASPARAM *p = NULL;
+
+	sscanf(possibleparam, "%s", param);
+
+	// Add Parameter to head of list
+	p = safe_malloc(sizeof(t_FASPARAM));
+	p->fasparam = safe_strdup(param);
+	p->next = config.fas_custom_parameters_list;
+
+	config.fas_custom_parameters_list = p;
+	debug(LOG_INFO, "Added Custom Parameter [%s]", possibleparam);
+	return 0;
+}
+
+int add_to_fas_custom_variables_list(const char possiblevar[])
+{
+	char var[512];
+	t_FASVAR *p = NULL;
+
+	sscanf(possiblevar, "%s", var);
+
+	// Add Variable to head of list
+	p = safe_malloc(sizeof(t_FASVAR));
+	p->fasvar = safe_strdup(var);
+	p->next = config.fas_custom_variables_list;
+
+	config.fas_custom_variables_list = p;
+	debug(LOG_INFO, "Added Custom Variable [%s]", possiblevar);
+	return 0;
+}
+
+int add_to_fas_custom_images_list(const char possibleimage[])
+{
+	char image[512];
+	t_FASIMG *p = NULL;
+
+	sscanf(possibleimage, "%s", image);
+
+	// Add Image to head of list
+	p = safe_malloc(sizeof(t_FASIMG));
+	p->fasimg = safe_strdup(image);
+	p->next = config.fas_custom_images_list;
+
+	config.fas_custom_images_list = p;
+	debug(LOG_INFO, "Added Custom Image [%s]", possibleimage);
+	return 0;
+}
+
+int add_to_fas_custom_files_list(const char possiblefile[])
+{
+	char file[512];
+	t_FASFILE *p = NULL;
+
+	sscanf(possiblefile, "%s", file);
+
+	// Add File to head of list
+	p = safe_malloc(sizeof(t_FASFILE));
+	p->fasfile = safe_strdup(file);
+	p->next = config.fas_custom_files_list;
+
+	config.fas_custom_files_list = p;
+	debug(LOG_INFO, "Added Custom File [%s]", possiblefile);
+	return 0;
+}
+
+
 int add_to_trusted_mac_list(const char possiblemac[])
 {
 	char mac[18];
@@ -1115,7 +1265,7 @@ int remove_from_trusted_mac_list(const char possiblemac[])
 
 	// check for valid format
 	if (!check_mac_format(possiblemac)) {
-		debug(LOG_NOTICE, "[%s] not a valid MAC address", possiblemac);
+		debug(LOG_ERR, "[%s] not a valid MAC address", possiblemac);
 		return -1;
 	}
 
@@ -1232,13 +1382,13 @@ int add_to_blocked_mac_list(const char possiblemac[])
 
 	// check for valid format
 	if (!check_mac_format(possiblemac)) {
-		debug(LOG_NOTICE, "[%s] not a valid MAC address to block", possiblemac);
+		debug(LOG_ERR, "[%s] not a valid MAC address to block", possiblemac);
 		return -1;
 	}
 
 	// abort if not using BLOCK mechanism
 	if (MAC_BLOCK != config.macmechanism) {
-		debug(LOG_NOTICE, "Attempt to access blocked MAC list but control mechanism != block");
+		debug(LOG_ERR, "Attempt to access blocked MAC list but control mechanism != block");
 		return -1;
 	}
 
@@ -1273,13 +1423,13 @@ int remove_from_blocked_mac_list(const char possiblemac[])
 
 	// check for valid format
 	if (!check_mac_format(possiblemac)) {
-		debug(LOG_NOTICE, "[%s] not a valid MAC address", possiblemac);
+		debug(LOG_ERR, "[%s] not a valid MAC address", possiblemac);
 		return -1;
 	}
 
 	// abort if not using BLOCK mechanism
 	if (MAC_BLOCK != config.macmechanism) {
-		debug(LOG_NOTICE, "Attempt to access blocked MAC list but control mechanism != block");
+		debug(LOG_ERR, "Attempt to access blocked MAC list but control mechanism != block");
 		return -1;
 	}
 
@@ -1343,13 +1493,13 @@ int add_to_allowed_mac_list(const char possiblemac[])
 
 	// check for valid format
 	if (!check_mac_format(possiblemac)) {
-		debug(LOG_NOTICE, "[%s] not a valid MAC address to allow", possiblemac);
+		debug(LOG_ERR, "[%s] not a valid MAC address to allow", possiblemac);
 		return -1;
 	}
 
 	// abort if not using ALLOW mechanism
 	if (MAC_ALLOW != config.macmechanism) {
-		debug(LOG_NOTICE, "Attempt to access allowed MAC list but control mechanism != allow");
+		debug(LOG_ERR, "Attempt to access allowed MAC list but control mechanism != allow");
 		return -1;
 	}
 
@@ -1384,13 +1534,13 @@ int remove_from_allowed_mac_list(const char possiblemac[])
 
 	// check for valid format
 	if (!check_mac_format(possiblemac)) {
-		debug(LOG_NOTICE, "[%s] not a valid MAC address", possiblemac);
+		debug(LOG_ERR, "[%s] not a valid MAC address", possiblemac);
 		return -1;
 	}
 
 	// abort if not using ALLOW mechanism
 	if (MAC_ALLOW != config.macmechanism) {
-		debug(LOG_NOTICE, "Attempt to access allowed MAC list but control mechanism != allow");
+		debug(LOG_ERR, "Attempt to access allowed MAC list but control mechanism != allow");
 		return -1;
 	}
 
@@ -1444,6 +1594,232 @@ void parse_allowed_mac_list(const char ptr[])
 	free(ptrcopyptr);
 }
 
+/* Given a pointer to a comma or whitespace delimited sequence of
+ * Walled Garden FQDNs, add each FQDN to config.walledgardenfqdnlist
+ */
+void parse_walledgarden_fqdn_list(const char ptr[])
+{
+	char *ptrcopy = NULL;
+	char *ptrcopyptr;
+	char *possiblefqdn = NULL;
+
+	debug(LOG_DEBUG, "Parsing string [%s] for Walled Garden FQDNs to allow", ptr);
+
+	// strsep modifies original, so let's make a copy
+	ptrcopyptr = ptrcopy = safe_strdup(ptr);
+
+	while ((possiblefqdn = strsep(&ptrcopy, ", \t"))) {
+		if (strlen(possiblefqdn) > 0) {
+			if (add_to_walledgarden_fqdn_list(possiblefqdn) < 0) {
+				exit(1);
+			}
+		}
+	}
+
+	free(ptrcopyptr);
+}
+
+/* Given a pointer to a comma or whitespace delimited sequence of
+ * Walled Garden FQDN ports, add each port number to config.walledgardenportlist
+ */
+void parse_walledgarden_port_list(const char ptr[])
+{
+	char *ptrcopy = NULL;
+	char *ptrcopyptr;
+	char *possibleport = NULL;
+
+	debug(LOG_DEBUG, "Parsing string [%s] for Walled Garden Ports to allow", ptr);
+
+	// strsep modifies original, so let's make a copy
+	ptrcopyptr = ptrcopy = safe_strdup(ptr);
+
+	while ((possibleport = strsep(&ptrcopy, ", \t"))) {
+		if (strlen(possibleport) > 0) {
+			if (add_to_walledgarden_port_list(possibleport) < 0) {
+				exit(1);
+			}
+		}
+	}
+
+	free(ptrcopyptr);
+}
+
+/* Given a pointer to a comma or whitespace delimited sequence of
+ * Custom FAS Parameters, add each parameter to config.fas_custom_parameters_list
+ */
+void parse_fas_custom_parameters_list(const char ptr[])
+{
+	char *ptrcopy = NULL;
+	char *possibleparam = NULL;
+	char msg[512] = {0};
+	char *cmd = NULL;
+	char possibleparam_urlencoded[512] = {0};
+
+	debug(LOG_INFO, "Parsing list [%s] for Custom FAS Parameters", ptr);
+
+	// strsep modifies original, so let's make a copy
+	ptrcopy = safe_strdup(ptr);
+
+	while ((possibleparam = strsep(&ptrcopy, ", \t"))) {
+		if (strlen(possibleparam) > 0) {
+
+			// URL encode Parameter before parsing
+			memset(possibleparam_urlencoded, 0, sizeof(possibleparam_urlencoded));
+			uh_urlencode(possibleparam_urlencoded, sizeof(possibleparam_urlencoded), possibleparam, strlen(possibleparam));
+			debug(LOG_DEBUG, "[%s] is the urlencoded Custom FAS Parameter", possibleparam_urlencoded);
+
+			safe_asprintf(&cmd,
+				"echo \"%s\" | awk -F'%s' 'NF>1 && $1!=\"\" && $2!=\"\" {printf \"%s\",$0}'",
+				possibleparam_urlencoded,
+				CUSTOM_SEPARATOR,
+				FORMAT_SPECIFIER
+			);
+			debug(LOG_DEBUG, "Parse command [%s]", cmd);
+			memset(msg, 0, sizeof(msg));
+			execute_ret_url_encoded(msg, sizeof(msg) - 1, cmd);
+			free(cmd);
+			if (strcmp(msg, possibleparam_urlencoded) == 0) {
+				debug(LOG_INFO, "Adding parameter [%s] [%s]", possibleparam, msg);
+				add_to_fas_custom_parameters_list(possibleparam);
+			} else {
+				debug(LOG_WARNING, "Invalid Custom Parameter [%s] [%s] - skipping", possibleparam, msg);
+			}
+		}
+	}
+}
+
+/* Given a pointer to a comma or whitespace delimited sequence of
+ * Custom FAS Variables, add each parameter to config.fas_custom_variables_list
+ */
+void parse_fas_custom_variables_list(const char ptr[])
+{
+	char *ptrcopy = NULL;
+	char *possiblevar = NULL;
+	char msg[512] = {0};
+	char *cmd = NULL;
+	char possiblevar_urlencoded[512] = {0};
+
+	debug(LOG_INFO, "Parsing list [%s] for Custom FAS Variables", ptr);
+
+	// strsep modifies original, so let's make a copy
+	ptrcopy = safe_strdup(ptr);
+
+	while ((possiblevar = strsep(&ptrcopy, ", \t"))) {
+		if (strlen(possiblevar) > 0) {
+
+			// URL encode Variable before parsing
+			memset(possiblevar_urlencoded, 0, sizeof(possiblevar_urlencoded));
+			uh_urlencode(possiblevar_urlencoded, sizeof(possiblevar_urlencoded), possiblevar, strlen(possiblevar));
+			debug(LOG_DEBUG, "[%s] is the urlencoded Custom FAS Variable", possiblevar_urlencoded);
+
+			safe_asprintf(&cmd,
+				"echo \"%s\" | awk -F'%s' 'NF>1 && $1!=\"\" && $2!=\"\" {printf \"%s\",$0}'",
+				possiblevar_urlencoded,
+				CUSTOM_SEPARATOR,
+				FORMAT_SPECIFIER
+			);
+			debug(LOG_DEBUG, "Parse command [%s]", cmd);
+			memset(msg, 0, sizeof(msg));
+			execute_ret_url_encoded(msg, sizeof(msg) - 1, cmd);
+			free(cmd);
+			if (strcmp(msg, possiblevar_urlencoded) == 0) {
+				debug(LOG_INFO, "Adding variable [%s] [%s]", possiblevar, msg);
+				add_to_fas_custom_variables_list(possiblevar);
+			} else {
+				debug(LOG_WARNING, "Invalid Custom Variable [%s] [%s] - skipping", possiblevar, msg);
+			}
+		}
+	}
+}
+
+/* Given a pointer to a comma or whitespace delimited sequence of
+ * Custom FAS Images, add each image to config.fas_custom_images_list
+ */
+void parse_fas_custom_images_list(const char ptr[])
+{
+	char *ptrcopy = NULL;
+	char *possibleimage = NULL;
+	char msg[512] = {0};
+	char *cmd = NULL;
+	char possibleimage_urlencoded[512] = {0};
+
+	debug(LOG_INFO, "Parsing list [%s] for Custom FAS Images", ptr);
+
+	// strsep modifies original, so let's make a copy
+	ptrcopy = safe_strdup(ptr);
+
+	while ((possibleimage = strsep(&ptrcopy, ", \t"))) {
+		if (strlen(possibleimage) > 0) {
+
+			// URL encode Image before parsing
+			memset(possibleimage_urlencoded, 0, sizeof(possibleimage_urlencoded));
+			uh_urlencode(possibleimage_urlencoded, sizeof(possibleimage_urlencoded), possibleimage, strlen(possibleimage));
+			debug(LOG_DEBUG, "[%s] is the urlencoded Custom FAS Image", possibleimage_urlencoded);
+
+			safe_asprintf(&cmd,
+				"echo \"%s\" | awk -F'%s' 'NF>1 && $1!=\"\" && $2!=\"\" {printf \"%s\",$0}'",
+				possibleimage_urlencoded,
+				CUSTOM_SEPARATOR,
+				FORMAT_SPECIFIER
+			);
+			debug(LOG_DEBUG, "Parse command [%s]", cmd);
+			memset(msg, 0, sizeof(msg));
+			execute_ret_url_encoded(msg, sizeof(msg) - 1, cmd);
+			free(cmd);
+			if (strcmp(msg, possibleimage_urlencoded) == 0) {
+				debug(LOG_INFO, "Adding image [%s] [%s]", possibleimage, msg);
+				add_to_fas_custom_images_list(possibleimage);
+			} else {
+				debug(LOG_WARNING, "Invalid Custom Image [%s] [%s] - skipping", possibleimage, msg);
+			}
+		}
+	}
+}
+
+/* Given a pointer to a comma or whitespace delimited sequence of
+ * Custom FAS Files, add each image to config.fas_custom_files_list
+ */
+void parse_fas_custom_files_list(const char ptr[])
+{
+	char *ptrcopy = NULL;
+	char *possiblefile = NULL;
+	char msg[512] = {0};
+	char *cmd = NULL;
+	char possiblefile_urlencoded[512] = {0};
+
+	debug(LOG_INFO, "Parsing list [%s] for Custom FAS Files", ptr);
+
+	// strsep modifies original, so let's make a copy
+	ptrcopy = safe_strdup(ptr);
+
+	while ((possiblefile = strsep(&ptrcopy, ", \t"))) {
+		if (strlen(possiblefile) > 0) {
+
+			// URL encode Image before parsing
+			memset(possiblefile_urlencoded, 0, sizeof(possiblefile_urlencoded));
+			uh_urlencode(possiblefile_urlencoded, sizeof(possiblefile_urlencoded), possiblefile, strlen(possiblefile));
+			debug(LOG_DEBUG, "[%s] is the urlencoded Custom FAS Image", possiblefile_urlencoded);
+
+			safe_asprintf(&cmd,
+				"echo \"%s\" | awk -F'%s' 'NF>1 && $1!=\"\" && $2!=\"\" {printf \"%s\",$0}'",
+				possiblefile_urlencoded,
+				CUSTOM_SEPARATOR,
+				FORMAT_SPECIFIER
+			);
+			debug(LOG_DEBUG, "Parse command [%s]", cmd);
+			memset(msg, 0, sizeof(msg));
+			execute_ret_url_encoded(msg, sizeof(msg) - 1, cmd);
+			free(cmd);
+			if (strcmp(msg, possiblefile_urlencoded) == 0) {
+				debug(LOG_INFO, "Adding file [%s] [%s]", possiblefile, msg);
+				add_to_fas_custom_files_list(possiblefile);
+			} else {
+				debug(LOG_WARNING, "Invalid Custom File [%s] [%s] - skipping", possiblefile, msg);
+			}
+		}
+	}
+}
+
 /** Set the debug log level.  See syslog.h
  *  Return 0 on success.
  */
