@@ -24,12 +24,12 @@
     @author Copyright (C) 2007 Paul Kube <nodogsplash@kokoro.ucsd.edu>
 */
 
-#define COPYRIGHT "openNDS, Copyright (C) 2020, The openNDS Contributors"
+#define COPYRIGHT "openNDS, Copyright (C) 2021, The openNDS Contributors"
 
 #ifndef _CONF_H_
 #define _CONF_H_
 
-#define VERSION "5.2.0"
+#define VERSION "9.0.0"
 
 /*
  * Defines how many times should we try detecting the interface with the default route (in seconds).
@@ -50,41 +50,40 @@
 #endif
 #define DEFAULT_DAEMON 1
 #define DEFAULT_DEBUGLEVEL 1
-#define DEFAULT_MAXCLIENTS 20
+#define DEFAULT_MAXCLIENTS 250
+#define DEFAULT_GATEWAYINTERFACE "br-lan"
 #define DEFAULT_GATEWAY_IPRANGE "0.0.0.0/0"
 #define DEFAULT_GATEWAYNAME "openNDS"
 #define DEFAULT_GATEWAYPORT 2050
+#define DEFAULT_GATEWAYFQDN "status.client"
 #define DEFAULT_FASPORT 0
 #define DEFAULT_LOGIN_OPTION_ENABLED 0
 #define DEFAULT_USE_OUTDATED_MHD 0
 #define DEFAULT_UNESCAPE_CALLBACK_ENABLED 0
 #define DEFAULT_FAS_SECURE_ENABLED 1
 #define DEFAULT_FASPATH "/"
-#define DEFAULT_REMOTE_AUTH_PORT 80
-#define DEFAULT_CHECKINTERVAL 30
-#define DEFAULT_SESSION_TIMEOUT 0
-#define DEFAULT_PREAUTH_IDLE_TIMEOUT 10
+#define DEFAULT_FASKEY "1234567890"
+#define DEFAULT_CHECKINTERVAL 60
+#define DEFAULT_SESSION_TIMEOUT 1200
+#define DEFAULT_PREAUTH_IDLE_TIMEOUT 30
 #define DEFAULT_AUTH_IDLE_TIMEOUT 120
 #define DEFAULT_WEBROOT "/etc/opennds/htdocs"
-#define DEFAULT_SPLASHPAGE "splash.html"
-#define DEFAULT_STATUSPAGE "status.html"
 #define DEFAULT_AUTHDIR "opennds_auth"
 #define DEFAULT_DENYDIR "opennds_deny"
 #define DEFAULT_PREAUTHDIR "opennds_preauth"
 #define DEFAULT_MACMECHANISM MAC_BLOCK
-#define DEFAULT_SET_MSS 1
-#define DEFAULT_MSS_VALUE 0
-#define DEFAULT_TRAFFIC_CONTROL 0
-#define DEFAULT_RATE_CHECK_WINDOW 2
-#define DEFAULT_UPLOAD_RATE 0
-#define DEFAULT_DOWNLOAD_RATE 0
-#define DEFAULT_UPLOAD_QUOTA 0
-#define DEFAULT_DOWNLOAD_QUOTA 0
+#define DEFAULT_SET_MSS 1 //allow setting the TCP Maximum Segment Size
+#define DEFAULT_MSS_VALUE 0 // value to set the MSS. 0 means use max possible ie clamp-mss-to-pmtu
+#define DEFAULT_TRAFFIC_CONTROL 0 // Defunct. TODO could fix
+#define DEFAULT_RATE_CHECK_WINDOW 2 // The data rate check moving average window size multiply this by CHECKINTERVAL to give window size in seconds
+#define DEFAULT_UPLOAD_RATE 0 // 0 means no limit
+#define DEFAULT_DOWNLOAD_RATE 0 // 0 means no limit
+#define DEFAULT_UPLOAD_QUOTA 0 // 0 means no limit
+#define DEFAULT_DOWNLOAD_QUOTA 0 // 0 means no limit
 #define DEFAULT_UPLOAD_IFB 0
 #define DEFAULT_LOG_SYSLOG 0
 #define DEFAULT_SYSLOG_FACILITY LOG_DAEMON
-#define DEFAULT_NDSCTL_SOCK "/tmp/ndsctl.sock"
-#define DEFAULT_INTERNAL_SOCK "/tmp/ndsctl.sock"
+#define DEFAULT_NDSCTL_SOCK "ndsctl.sock"
 #define DEFAULT_FW_MARK_AUTHENTICATED 0x30000
 #define DEFAULT_FW_MARK_TRUSTED 0x20000
 #define DEFAULT_FW_MARK_BLOCKED 0x10000
@@ -132,69 +131,114 @@ typedef struct _MAC_t {
 	struct _MAC_t *next;
 } t_MAC;
 
+// Walled Garden Ports
+typedef struct _WGP_t {
+	unsigned int wgport;
+	struct _WGP_t *next;
+} t_WGP;
+
+// Walled Garden FQDNs
+typedef struct _WGFQDN_t {
+	char *wgfqdn;
+	struct _WGFQDN_t *next;
+} t_WGFQDN;
+
+// Custom FAS Parameters
+typedef struct _FASPARAM_t {
+	char *fasparam;
+	struct _FASPARAM_t *next;
+} t_FASPARAM;
+
+// Custom FAS Variables
+typedef struct _FASVAR_t {
+	char *fasvar;
+	struct _FASVAR_t *next;
+} t_FASVAR;
+
+// Custom FAS Images
+typedef struct _FASIMG_t {
+	char *fasimg;
+	struct _FASIMG_t *next;
+} t_FASIMG;
+
+// Custom FAS Files
+typedef struct _FASFILE_t {
+	char *fasfile;
+	struct _FASFILE_t *next;
+} t_FASFILE;
 
 // Configuration structure
 typedef struct {
-	char configfile[255];			//@brief name of the config file
-	char *ndsctl_sock;			//@brief ndsctl path to socket
-	char *internal_sock;			//@brief internal path to socket
-	int daemon;				//@brief if daemon > 0, use daemon mode
+	char configfile[255];				//@brief name of the config file
+	char *ndsctl_sock;				//@brief ndsctl path to socket
+	char *internal_sock;				//@brief internal path to socket
+	int daemon;					//@brief if daemon > 0, use daemon mode
 	int debuglevel;				//@brief Debug information verbosity
 	int maxclients;				//@brief Maximum number of clients allowed
-	char *gw_name;				//@brief Name of the gateway; e.g. its SSID or a unique identifier for use in a remote FAS
-	char *http_encoded_gw_name;		//@brief http encoded name of the gateway, used as a templated variable in splash.htm
-	char *url_encoded_gw_name;		//@brief url encoded name of the gateway used as variable in Preauth
-	char *gw_interface;			//@brief Interface we will manage
-	char *gw_iprange;			//@brief IP range on gw_interface we will manage
-	char *gw_ip;				//@brief Internal IP (v4 or v6) for our web server
-	char *gw_address;			//@brief Internal IP with port for our web server
-	char *gw_mac;				//@brief MAC address of the interface we manage
-	unsigned int gw_port;			//@brief Port the webserver will run on
-	unsigned int fas_port;			//@brief Port the fas server will run on
-	int login_option_enabled;		//@brief Use default PreAuth Login script
-	int use_outdated_mhd;			//@brief Use outdated libmicrohttpd
-	int unescape_callback_enabled;		//@brief Enable external MHD unescape callback script
+	char *gw_name;					//@brief Name of the gateway; e.g. its SSID or a unique identifier for use in a remote FAS
+	char *http_encoded_gw_name;			//@brief http encoded name of the gateway, used as a templated variable in splash.htm
+	char *url_encoded_gw_name;			//@brief url encoded name of the gateway used as variable in Preauth
+	char *gw_interface;				//@brief Interface we will manage
+	char *gw_iprange;				//@brief IP range on gw_interface we will manage
+	char *gw_ip;					//@brief Internal IP (v4 or v6) for our web server
+	char *gw_address;				//@brief Internal IP with port for our web server
+	char *gw_mac;					//@brief MAC address of the interface we manage
+	char *gw_fqdn;					//@brief FQDN of the client status page
+	unsigned int gw_port;				//@brief Port the webserver will run on
+	unsigned int fas_port;				//@brief Port the fas server will run on
+	int login_option_enabled;			//@brief Use default PreAuth Login script
+	int use_outdated_mhd;				//@brief Use outdated libmicrohttpd
+	int unescape_callback_enabled;			//@brief Enable external MHD unescape callback script
 	int fas_secure_enabled;			//@brief Enable Secure FAS
 	char *fas_path;				//@brief Path to forward authentication page of FAS
-	char *fas_key;				//@brief AES key for FAS
-	char *fas_remoteip;			//@brief IP addess of a remote FAS
-	char *fas_remotefqdn;			//@brief FQDN of a remote FAS
-	char *fas_url;				//@brief URL of a remote FAS
-	char *fas_ssl;				//@brief SSL provider for FAS
-	char *fas_hid;				//@brief Hash provider for FAS
-	char *webroot;				//@brief Directory containing splash pages, etc.
-	char *splashpage;			//@brief Name of main splash page
-	char *statuspage;			//@brief Name of info status page
-	char *redirectURL;			//@brief URL to direct client to after authentication
-	char *authdir;				//@brief Notional relative dir for authentication URL
-	char *denydir;				//@brief Notional relative dir for denial URL
-	char *preauthdir;			//@brief Notional relative dir for preauth URL
-	int session_timeout;			//@brief Minutes of the default session length
-	int preauth_idle_timeout;		//@brief Minutes a preauthenticated client will be kept in the system
-	int auth_idle_timeout;			//@brief Minutes an authenticated client will be kept in the system
-	int checkinterval;			//@brief Period the the client timeout check thread will run, in seconds
-	int set_mss;				//@brief boolean, whether to set mss
-	int mss_value;				//@brief int, mss value; <= 0 clamp to pmtu
-	int traffic_control;			//@brief boolean, whether to do tc
-	int rate_check_window;			//@brief window size in multiples of checkinterval for rate check moving average
-	unsigned long long int download_rate;	//@brief Download rate, kb/s
-	unsigned long long int upload_rate;	//@brief Upload rate, kb/s
-	unsigned long long int download_quota;	//@brief Download quota, kB
-	unsigned long long int upload_quota;	//@brief Upload quota, kB
+	char *fas_key;					//@brief AES key for FAS
+	char *fas_remoteip;				//@brief IP addess of a remote FAS
+	char *fas_remotefqdn;				//@brief FQDN of a remote FAS
+	char *fas_url;					//@brief URL of a remote FAS
+	char *fas_ssl;					//@brief SSL provider for FAS
+	char *fas_hid;					//@brief Hash provider for FAS
+	char *themespec_path;				//@brief Path to the ThemeSpec file to use for login_option_enabled = 3
+	char *tmpfsmountpoint;				//@brief Mountpoint of the tmpfs drive eg /tmp etc.
+	char *webroot;					//@brief Directory containing splash pages, etc.
+	char *authdir;					//@brief Notional relative dir for authentication URL
+	char *denydir;					//@brief Notional relative dir for denial URL
+	char *preauthdir;				//@brief Notional relative dir for preauth URL
+	int session_timeout;				//@brief Minutes of the default session length
+	int preauth_idle_timeout;			//@brief Minutes a preauthenticated client will be kept in the system
+	int auth_idle_timeout;				//@brief Minutes an authenticated client will be kept in the system
+	int checkinterval;				//@brief Period the the client timeout check thread will run, in seconds
+	int set_mss;					//@brief boolean, whether to set mss
+	int mss_value;					//@brief int, mss value; <= 0 clamp to pmtu
+	int traffic_control;				//@brief boolean, whether to do tc
+	int rate_check_window;				//@brief window size in multiples of checkinterval for rate check moving average
+	unsigned long long int download_rate;		//@brief Download rate, kb/s
+	unsigned long long int upload_rate;		//@brief Upload rate, kb/s
+	unsigned long long int download_quota;		//@brief Download quota, kB
+	unsigned long long int upload_quota;		//@brief Upload quota, kB
 	int upload_ifb;				//@brief Number of IFB handling upload
 	int log_syslog;				//@brief boolean, whether to log to syslog
-	int syslog_facility;			//@brief facility to use when using syslog for logging
-	int macmechanism; 			//@brief mechanism wrt MAC addrs
-	t_firewall_ruleset *rulesets;		//@brief firewall rules
-	t_MAC *trustedmaclist;			//@brief list of trusted macs
-	t_MAC *blockedmaclist;			//@brief list of blocked macs
-	t_MAC *allowedmaclist;			//@brief list of allowed macs
-	unsigned int fw_mark_authenticated;	//@brief iptables mark for authenticated packets
-	unsigned int fw_mark_blocked;		//@brief iptables mark for blocked packets
-	unsigned int fw_mark_trusted;		//@brief iptables mark for trusted packets
-	int ip6;				//@brief enable IPv6
-	char *binauth;				//@brief external authentication program
-	char *preauth;				//@brief external preauthentication program
+	int syslog_facility;				//@brief facility to use when using syslog for logging
+	int macmechanism; 				//@brief mechanism wrt MAC addrs
+	t_firewall_ruleset *rulesets;			//@brief firewall rules
+	t_MAC *trustedmaclist;				//@brief list of trusted macs
+	t_MAC *blockedmaclist;				//@brief list of blocked macs
+	t_MAC *allowedmaclist;				//@brief list of allowed macs
+	t_WGP *walledgarden_port_list;			//@brief list of Walled Garden Ports
+	t_WGFQDN *walledgarden_fqdn_list;		//@brief list of Walled Garden FQDNs
+	t_FASPARAM *fas_custom_parameters_list;	//@brief list of Custom FAS parameters
+	t_FASVAR *fas_custom_variables_list;		//@brief list of Custom FAS variables
+	t_FASIMG *fas_custom_images_list;		//@brief list of Custom FAS images
+	t_FASFILE *fas_custom_files_list;		//@brief list of Custom FAS files
+	char *custom_params;				//@brief FAS custom parameter string
+	char *custom_vars;				//@brief FAS custom variable string
+	char *custom_images;				//@brief FAS custom image string
+	char *custom_files;				//@brief FAS custom file string
+	unsigned int fw_mark_authenticated;		//@brief iptables mark for authenticated packets
+	unsigned int fw_mark_blocked;			//@brief iptables mark for blocked packets
+	unsigned int fw_mark_trusted;			//@brief iptables mark for trusted packets
+	int ip6;					//@brief enable IPv6
+	char *binauth;					//@brief external authentication program
+	char *preauth;					//@brief external preauthentication program
 } s_config;
 
 // @brief Get the current gateway configuration
@@ -230,6 +274,12 @@ char * get_empty_ruleset_policy(const char[]);
 void parse_trusted_mac_list(const char[]);
 void parse_blocked_mac_list(const char[]);
 void parse_allowed_mac_list(const char[]);
+void parse_walledgarden_fqdn_list(const char[]);
+void parse_walledgarden_port_list(const char[]);
+void parse_fas_custom_parameters_list(const char[]);
+void parse_fas_custom_variables_list(const char[]);
+void parse_fas_custom_images_list(const char[]);
+void parse_fas_custom_files_list(const char[]);
 
 int is_blocked_mac(const char *mac);
 int is_allowed_mac(const char *mac);
