@@ -66,18 +66,17 @@
 #include <microhttpd.h>
 
 /* Check for libmicrohttp version in compiler
- *0.9.51 is the minimum version for NDS to work
+ *0.9.71 is the minimum version for NDS to work with the new API
  */
 #if MHD_VERSION < 0x00095100
-#error libmicrohttp version >= 0.9.51 required
+#error libmicrohttp version >= 0.9.71 required
 #endif
 /* Check for libmicrohttp version at runtime
  *0.9.69 is the minimum version to prevent loss of special characters in form data (BinAuth and PreAuth) 
  */
 #define MIN_MHD_MAJOR 0
 #define MIN_MHD_MINOR 9
-#define MIN_MHD_PATCH 69
-#define MAX_MHD_PATCH 70
+#define MIN_MHD_PATCH 71
 
 /** XXX Ugly hack
  * We need to remember the thread IDs of threads that simulate wait with pthread_cond_timedwait
@@ -85,9 +84,6 @@
  */
 static pthread_t tid_client_check = 0;
 
-// The internal web server
-struct MHD_Daemon * webserver = NULL;
-
 // Time when opennds started
 time_t started_time = 0;
 
@@ -140,7 +136,7 @@ termination_handler(int s)
 	static pthread_mutex_t sigterm_mutex = PTHREAD_MUTEX_INITIALIZER;
 	char *fasssl = NULL;
 
-	debug(LOG_NOTICE, "Handler for termination caught signal %d", s);
+	debug(LOG_INFO, "Handler for termination caught signal %d", s);
 
 	// Makes sure we only call iptables_fw_destroy() once.
 	if (pthread_mutex_trylock(&sigterm_mutex)) {
@@ -244,21 +240,30 @@ setup_from_config(void)
 	char msg[255] = {0};
 	char gwhash[255] = {0};
 	char authmonpid[255] = {0};
+	char *socket;
 	char *fasurl = NULL;
 	char *fasssl = NULL;
 	char *gatewayhash = NULL;
 	char *fashid = NULL;
 	char *phpcmd = NULL;
 	char *preauth_dir = NULL;
-	char loginscript[] = "/usr/lib/opennds/login.sh";
+	char loginscript[] = "/usr/lib/opennds/libopennds.sh";
 	char gw_name_entityencoded[QUERYMAXLEN] = {0};
 	char gw_name_urlencoded[QUERYMAXLEN] = {0};
 	struct stat sb;
 	time_t sysuptime;
+	t_WGFQDN *allowed_wgfqdn;
+	char wgfqdns[1024] = {0};
+	char *dnscmd;
+
 	s_config *config;
 
 	config = config_get_config();
 
+	debug(LOG_INFO, "tmpfs mountpoint is [%s]", config->tmpfsmountpoint);
+	// Before we do anything else, reset the firewall (cleans it, in case we are restarting after opennds crash)
+	iptables_fw_destroy();
+
 	// Check for libmicrohttp version at runtime, ie actual installed version
 	int major = 0;
 	int minor = 0;
@@ -278,55 +283,19 @@ setup_from_config(void)
 
 		} else if (patch < MIN_MHD_PATCH) {
 			outdated = 1;
-
-		} else if (patch > MAX_MHD_PATCH) {
-			outdated = 2;
 		}
 
 		if (outdated == 1) {
-			debug(LOG_ERR, "libmicrohttpd is out of date, please upgrade to v%d.%d.%d or v%d.%d.%d",
-				MIN_MHD_MAJOR, MIN_MHD_MINOR, MIN_MHD_PATCH,
-				MIN_MHD_MAJOR, MIN_MHD_MINOR, MAX_MHD_PATCH
-			);
+			debug(LOG_ERR, "libmicrohttpd is out of date, please upgrade to version %d.%d.%d or higher",
+				MIN_MHD_MAJOR, MIN_MHD_MINOR, MIN_MHD_PATCH);
 
 			if (config->use_outdated_mhd == 0) {
 				debug(LOG_ERR, "exiting...");
 				exit(1);
 			} else {
-				debug(LOG_ERR, "Client entered data may be translated incorrectly, use with this possibility in mind...");
+				debug(LOG_ERR, "Attempting use of outdated MHD - Data may be corrupted or openNDS may fail...");
 			}
 		}
-
-		if (outdated == 2) {
-			debug(LOG_ERR, "libmicrohttpd is too new, please downgrade to v%d.%d.%d or v%d.%d.%d",
-				MIN_MHD_MAJOR, MIN_MHD_MINOR, MIN_MHD_PATCH,
-				MIN_MHD_MAJOR, MIN_MHD_MINOR, MAX_MHD_PATCH
-			);
-
-			debug(LOG_ERR, "exiting...");
-			exit(1);
-		}
-
-
-	}
-
-	// Encode gatewayname
-	htmlentityencode(gw_name_entityencoded, sizeof(gw_name_entityencoded), config->gw_name, strlen(config->gw_name));
-	config->http_encoded_gw_name = gw_name_entityencoded;
-
-	uh_urlencode(gw_name_urlencoded, sizeof(gw_name_urlencoded), config->gw_name, strlen(config->gw_name));
-	config->url_encoded_gw_name = gw_name_urlencoded;
-
-	// Set the time when opennds started
-	sysuptime = get_system_uptime ();
-	debug(LOG_INFO, "main: System Uptime is %li seconds", sysuptime);
-
-	if (!started_time) {
-		debug(LOG_INFO, "Setting started_time");
-		started_time = time(NULL);
-	} else if (started_time < (time(NULL) - sysuptime)) {
-		debug(LOG_WARNING, "Detected possible clock skew - re-setting started_time");
-		started_time = time(NULL);
 	}
 
 	// If we don't have the Gateway IP address, get it. Exit on failure.
@@ -359,63 +328,219 @@ setup_from_config(void)
 		config->fas_remoteip = safe_strdup(config->gw_ip);
 	}
 
-	// Initializes the web server
-	if (config->unescape_callback_enabled == 0) {
-		debug(LOG_INFO, "MHD Unescape Callback is Disabled");
+	// Setup custom FAS parameters if configured
+	char fasparam[MAX_BUF] = {0};
+	t_FASPARAM *fas_fasparam;
+	if (config->fas_custom_parameters_list) {
+		for (fas_fasparam = config->fas_custom_parameters_list; fas_fasparam != NULL; fas_fasparam = fas_fasparam->next) {
 
-		if ((webserver = MHD_start_daemon(MHD_USE_EPOLL_INTERNALLY | MHD_USE_TCP_FASTOPEN,
-								config->gw_port,
-								NULL, NULL,
-								libmicrohttpd_cb, NULL,
-								MHD_OPTION_CONNECTION_TIMEOUT, (unsigned int) 120,
-								MHD_OPTION_LISTENING_ADDRESS_REUSE, 1,
-								MHD_OPTION_END)) == NULL) {
-			debug(LOG_ERR, "Could not create web server: %s", strerror(errno));
-			exit(1);
-		}
-
-	} else {
-		debug(LOG_NOTICE, "MHD Unescape Callback is Enabled");
-
-		if ((webserver = MHD_start_daemon(MHD_USE_EPOLL_INTERNALLY | MHD_USE_TCP_FASTOPEN,
-								config->gw_port,
-								NULL, NULL,
-								libmicrohttpd_cb, NULL,
-								MHD_OPTION_CONNECTION_TIMEOUT, (unsigned int) 120,
-								MHD_OPTION_LISTENING_ADDRESS_REUSE, 1,
-								MHD_OPTION_UNESCAPE_CALLBACK, unescape, NULL,
-								MHD_OPTION_END)) == NULL) {
-			debug(LOG_ERR, "Could not create web server: %s", strerror(errno));
-			exit(1);
+			// Make sure we don't have a buffer overflow
+			if ((sizeof(fasparam) - strlen(fasparam)) > (strlen(fas_fasparam->fasparam) + 4)) {
+				strcat(fasparam, fas_fasparam->fasparam);
+				strcat(fasparam, QUERYSEPARATOR);
+			} else {
+				break;
+			}
 		}
+		config->custom_params = safe_strdup(fasparam);
+		debug(LOG_DEBUG, "Custom FAS parameter string [%s]", config->custom_params);
 	}
 
-	// TODO: set listening socket - do we need it?
+	// Setup custom FAS variables if configured
+	char fasvar[MAX_BUF] = {0};
+	t_FASVAR *fas_fasvar;
+	if (config->fas_custom_parameters_list) {
+		for (fas_fasvar = config->fas_custom_variables_list; fas_fasvar != NULL; fas_fasvar = fas_fasvar->next) {
 
+			// Make sure we don't have a buffer overflow
+			if ((sizeof(fasvar) - strlen(fasvar)) > (strlen(fas_fasvar->fasvar) + 4)) {
+				strcat(fasvar, fas_fasvar->fasvar);
+				strcat(fasvar, QUERYSEPARATOR);
+			} else {
+				break;
+			}
+		}
+		config->custom_vars = safe_strdup(fasvar);
+		debug(LOG_DEBUG, "Custom FAS variables string [%s]", config->custom_vars);
+	}
+
+	// Setup custom FAS images if configured
+	char fasimage[MAX_BUF] = {0};
+	t_FASIMG *fas_fasimage;
+	if (config->fas_custom_images_list) {
+		for (fas_fasimage = config->fas_custom_images_list; fas_fasimage != NULL; fas_fasimage = fas_fasimage->next) {
+
+			// Make sure we don't have a buffer overflow
+			if ((sizeof(fasimage) - strlen(fasimage)) > (strlen(fas_fasimage->fasimg) + 4)) {
+				strcat(fasimage, fas_fasimage->fasimg);
+				strcat(fasimage, QUERYSEPARATOR);
+			} else {
+				break;
+			}
+		}
+		config->custom_images = safe_strdup(fasimage);
+		debug(LOG_DEBUG, "Custom FAS images string [%s]", config->custom_images);
+	}
+
+	// Setup custom FAS images if configured
+	char fasfile[MAX_BUF] = {0};
+	t_FASFILE *fas_fasfile;
+	if (config->fas_custom_files_list) {
+		for (fas_fasfile = config->fas_custom_files_list; fas_fasfile != NULL; fas_fasfile = fas_fasfile->next) {
+
+			// Make sure we don't have a buffer overflow
+			if ((sizeof(fasfile) - strlen(fasfile)) > (strlen(fas_fasfile->fasfile) + 4)) {
+				strcat(fasfile, fas_fasfile->fasfile);
+				strcat(fasfile, QUERYSEPARATOR);
+			} else {
+				break;
+			}
+		}
+		config->custom_files = safe_strdup(fasfile);
+		debug(LOG_DEBUG, "Custom FAS files string [%s]", config->custom_files);
+	}
+
+	if (strcmp(config->gw_fqdn, "disable") == 0) {
+		config->gw_fqdn = NULL;
+	}
+
+	if (config->gw_fqdn || config->walledgarden_fqdn_list) {
+		// For Client status Page - configure the hosts file
+		if (config->gw_fqdn) {
+			safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"hostconf\" \"%s\" \"%s\"",
+				config->gw_ip,
+				config->gw_fqdn
+			);
+
+			if (execute_ret_url_encoded(msg, sizeof(msg) - 1, dnscmd) == 0) {
+				debug(LOG_INFO, "Dnsmasq configured for Walled Garden");
+			} else {
+				debug(LOG_ERR, "Client Status Page: Hosts setup script failed to execute");
+			}
+			free(dnscmd);
+		}
+
+		// For Walled Garden - Check we have ipset support and if we do, set it up
+		if (config->walledgarden_fqdn_list) {
+			// Check ipset command is available
+			if (execute_ret_url_encoded(msg, sizeof(msg) - 1, "ipset -v") == 0) {
+				debug(LOG_NOTICE, "ipset support is available");
+			} else {
+				debug(LOG_ERR, "ipset support not available - please install package to provide it");
+				debug(LOG_ERR, "Exiting...");
+				exit(1);
+			}
+
+			// Check we have dnsmasq ipset compile option
+			if (execute_ret_url_encoded(msg, sizeof(msg) - 1, "dnsmasq --version | grep ' ipset '") == 0) {
+				debug(LOG_NOTICE, "dnsmasq ipset support is available");
+			} else {
+				debug(LOG_ERR, "Please install dnsmasq full version with ipset compile option");
+				debug(LOG_ERR, "Exiting...");
+				exit(1);
+			}
+
+			// If Walled Garden ipset exists, destroy it.
+			execute_ret_url_encoded(msg, sizeof(msg) - 1, "ipset destroy walledgarden");
+
+			// Set up the Walled Garden
+			if (execute_ret_url_encoded(msg, sizeof(msg) - 1, "ipset create walledgarden hash:ip") == 0) {
+				debug(LOG_INFO, "Walled Garden ipset created");
+			} else {
+				debug(LOG_ERR, "Failed to create Walled Garden");
+				debug(LOG_ERR, "Exiting...");
+				exit(1);
+			}
+
+			// Configure dnsmasq
+			for (allowed_wgfqdn = config->walledgarden_fqdn_list; allowed_wgfqdn != NULL; allowed_wgfqdn = allowed_wgfqdn->next) {
+
+				// Make sure we don't have a buffer overflow
+				if ((sizeof(wgfqdns) - strlen(wgfqdns)) > (strlen(allowed_wgfqdn->wgfqdn) + 15)) {
+					strcat(wgfqdns, "/");
+					strcat(wgfqdns, allowed_wgfqdn->wgfqdn);
+				} else {
+					break;
+				}
+			}
+
+			strcat(wgfqdns, "/walledgarden");
+			debug(LOG_DEBUG, "Dnsmasq Walled Garden config [%s]", wgfqdns);
+			safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"ipsetconf\" \"%s\"", wgfqdns);
+			if (execute_ret_url_encoded(msg, sizeof(msg) - 1, dnscmd) == 0) {
+				debug(LOG_INFO, "Dnsmasq configured for Walled Garden");
+			} else {
+				debug(LOG_ERR, "Walled Garden Dnsmasq setup script failed to execute");
+			}
+			free(dnscmd);
+		}
+
+		safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"restart_only\" &");
+		debug(LOG_DEBUG, "restart command [ %s ]", dnscmd);
+		system(dnscmd);
+		debug(LOG_INFO, "Dnsmasq restarted");
+		free(dnscmd);
+	}
+
+	// Encode gatewayname
+	htmlentityencode(gw_name_entityencoded, sizeof(gw_name_entityencoded), config->gw_name, strlen(config->gw_name));
+	config->http_encoded_gw_name = safe_strdup(gw_name_entityencoded);
+
+	uh_urlencode(gw_name_urlencoded, sizeof(gw_name_urlencoded), config->gw_name, strlen(config->gw_name));
+	config->url_encoded_gw_name = safe_strdup(gw_name_urlencoded);
+
+	// Set the time when opennds started
+	sysuptime = get_system_uptime ();
+	debug(LOG_INFO, "main: System Uptime is %li seconds", sysuptime);
+
+	if (!started_time) {
+		debug(LOG_INFO, "Setting started_time");
+		started_time = time(NULL);
+	} else if (started_time < (time(NULL) - sysuptime)) {
+		debug(LOG_WARNING, "Detected possible clock skew - re-setting started_time");
+		started_time = time(NULL);
+	}
+
+	// Initialize the web server
+	start_mhd();
 	debug(LOG_NOTICE, "Created web server on %s", config->gw_address);
-	debug(LOG_NOTICE, "Handle [%i]", webserver);
 
-	// If login script is enabled, check if the script actually exists
-	if (config->login_option_enabled > 0) {
-		debug(LOG_NOTICE, "Login option is Enabled.\n");
-
-		if (!((stat(loginscript, &sb) == 0) && S_ISREG(sb.st_mode) && (sb.st_mode & S_IXUSR))) {
-			debug(LOG_ERR, "Login script does not exist or is not executable: %s", loginscript);
-			debug(LOG_ERR, "Exiting...");
-			exit(1);
-		} else {
-			config->preauth = loginscript;
-		}
-
-	} else {
-		debug(LOG_INFO, "Using config options for FAS or Templated Splash.\n");
+	// Get the ndsctl socket path/filename
+	if (strcmp(config->ndsctl_sock, DEFAULT_NDSCTL_SOCK) == 0) {
+		safe_asprintf(&socket, "%s/%s", config->tmpfsmountpoint, DEFAULT_NDSCTL_SOCK);
+		config->ndsctl_sock = safe_strdup(socket);
+		free(socket);
 	}
 
-	// If PreAuth is enabled, override any FAS configuration
+	debug(LOG_NOTICE, "Socket access at %s", config->ndsctl_sock);
+
+	// Check if login script or custom preauth script is enabled
+	if (config->login_option_enabled >= 1) {
+		debug(LOG_NOTICE, "Login option is Enabled using mode %d.\n", config->login_option_enabled);
+		config->preauth = safe_strdup(loginscript);
+	} else if (config->login_option_enabled == 0 && config->fas_port == 0 && config->preauth == NULL) {
+		debug(LOG_NOTICE, "Click to Continue option is Enabled.\n");
+		config->preauth = safe_strdup(loginscript);
+	} else if (config->login_option_enabled == 0 && config->fas_port == 0 && config->preauth != NULL) {
+		debug(LOG_NOTICE, "Custom PreAuth Script Enabled.\n");
+	} else if (config->login_option_enabled == 0 && config->fas_port >= 1 ) {
+		debug(LOG_NOTICE, "FAS Enabled.\n");
+		config->preauth = NULL;
+	}
+
+
+	// If PreAuth is enabled, override any FAS configuration and check script exists
 	if (config->preauth) {
 		debug(LOG_NOTICE, "Preauth is Enabled - Overiding FAS configuration.\n");
 		debug(LOG_INFO, "Preauth Script is %s\n", config->preauth);
 
+
+		if (!((stat(config->preauth, &sb) == 0) && S_ISREG(sb.st_mode) && (sb.st_mode & S_IXUSR))) {
+			debug(LOG_ERR, "Login script does not exist or is not executable: %s", config->preauth);
+			debug(LOG_ERR, "Exiting...");
+			exit(1);
+		}
+
 		//override all other FAS settings
 		config->fas_remoteip = safe_strdup(config->gw_ip);
 		config->fas_remotefqdn = NULL;
@@ -452,8 +577,8 @@ setup_from_config(void)
 
 		// If FAS key is set, then check the prerequisites
 
-		// FAS secure Level 1
-		if (config->fas_key && config->fas_secure_enabled == 1) {
+		// FAS secure Level >=1
+		if (config->fas_key && config->fas_secure_enabled >= 1) {
 			// Check sha256sum command is available
 			if (execute_ret_url_encoded(msg, sizeof(msg) - 1, "printf 'test' | sha256sum") == 0) {
 				safe_asprintf(&fashid, "sha256sum");
@@ -522,17 +647,18 @@ setup_from_config(void)
 		debug(LOG_NOTICE, "FAS URL is %s\n", config->fas_url);
 		free(fasurl);
 
+		// Check if authmon is running and if it is, kill it
+		safe_asprintf(&fasssl, "kill $(pgrep -f \"usr/lib/opennds/authmon.sh\") > /dev/null 2>&1");
+		system(fasssl);
+		free(fasssl);
+
 		// Start the authmon daemon if configured for Level 3
 		if (config->fas_key && config->fas_secure_enabled == 3) {
-			// Check if authmon is already running and if it is, kill it
-			safe_asprintf(&fasssl, "kill $(pgrep -f \"usr/lib/opennds/authmon.sh\") > /dev/null 2>&1");
-			system(fasssl);
-			free(fasssl);
 
 			// Get the sha256 digest of gatewayname
 			safe_asprintf(&fasssl,
 				"echo \"<?php echo openssl_digest('%s', 'sha256'); ?>\" | %s",
-				config->gw_name,
+				config->url_encoded_gw_name,
 				config->fas_ssl
 			);
 
@@ -600,16 +726,15 @@ setup_from_config(void)
 		debug(LOG_INFO, "Binauth Script is %s\n", config->binauth);
 	}
 
-	// Reset the firewall (cleans it, in case we are restarting after opennds crash)
-	iptables_fw_destroy();
-
-	// Then initialize it
+	// Now initialize the firewall
 	if (iptables_fw_init() != 0) {
 		debug(LOG_ERR, "Error initializing firewall rules! Cleaning up");
 		iptables_fw_destroy();
 		debug(LOG_ERR, "Exiting because of error initializing firewall rules");
 		exit(1);
 	}
+
+	debug(LOG_NOTICE, "openNDS is now running.\n");
 }
 
 /**@internal
@@ -646,7 +771,9 @@ main_loop(void)
 	if (result) {
 		debug(LOG_INFO, "Failed to wait for opennds thread.");
 	}
-	MHD_stop_daemon(webserver);
+	//MHD_stop_daemon(webserver);
+	stop_mhd();
+
 	termination_handler(result);
 }
 
@@ -661,7 +788,7 @@ int main(int argc, char **argv)
 	parse_commandline(argc, argv);
 
 	// Initialize the config
-	debug(LOG_NOTICE, "openNDS Version %s \n", VERSION);
+	debug(LOG_NOTICE, "openNDS Version %s is in startup\n", VERSION);
 	debug(LOG_INFO, "Reading and validating configuration file %s", config->configfile);
 	config_read(config->configfile);
 	config_validate();
