@@ -28,6 +28,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <pthread.h>
 #include <string.h>
 #include <sys/types.h>
@@ -36,9 +37,13 @@
 #include <unistd.h>
 #include <syslog.h>
 #include <errno.h>
+#include <ctype.h>
 
 #include "ndsctl.h"
+#include "common.h"
 
+int safe_asprintf(char **strp, const char *fmt, ...);
+int safe_vasprintf(char **strp, const char *fmt, va_list ap);
 
 struct argument {
 	const char *cmd;
@@ -46,6 +51,76 @@ struct argument {
 	const char *ifno;
 };
 
+int safe_asprintf(char **strp, const char *fmt, ...)
+{
+	va_list ap;
+	int retval;
+
+	va_start(ap, fmt);
+	retval = safe_vasprintf(strp, fmt, ap);
+	va_end(ap);
+
+	return (retval);
+}
+
+int safe_vasprintf(char **strp, const char *fmt, va_list ap)
+{
+	int retval;
+
+	retval = vasprintf(strp, fmt, ap);
+
+	if (retval == -1) {
+		printf("Failed: Memory allocation error");
+		exit (1);
+	}
+
+	return (retval);
+}
+
+int uh_b64decode(char *buf, int blen, const void *src, int slen)
+{
+	const unsigned char *str = src;
+	unsigned int cout = 0;
+	unsigned int cin = 0;
+	int len = 0;
+	int i = 0;
+
+	for (i = 0; (i <= slen) && (str[i] != 0); i++) {
+		cin = str[i];
+
+		if ((cin >= '0') && (cin <= '9'))
+			cin = cin - '0' + 52;
+		else if ((cin >= 'A') && (cin <= 'Z'))
+			cin = cin - 'A';
+		else if ((cin >= 'a') && (cin <= 'z'))
+			cin = cin - 'a' + 26;
+		else if (cin == '+')
+			cin = 62;
+		else if (cin == '/')
+			cin = 63;
+		else if (cin == '=')
+			cin = 0;
+		else
+			continue;
+
+		cout = (cout << 6) | cin;
+
+		if ((i % 4) != 3)
+			continue;
+
+		if ((len + 3) >= blen)
+			break;
+
+		buf[len++] = (char)(cout >> 16);
+		buf[len++] = (char)(cout >> 8);
+		buf[len++] = (char)(cout);
+	}
+
+	//debug(LOG_DEBUG, "b64 decoded string: %s, decoded length: %d", buf, len);
+	buf[len++] = 0;
+	return len;
+}
+
 /** @internal
  * @brief Print usage
  *
@@ -64,8 +139,6 @@ usage(void)
 		"commands:\n"
 		"  status\n"
 		"	View the status of opennds\n\n"
-		"  clients\n"
-		"	Display machine-readable client list\n\n"
 		"  json	mac|ip|token(optional)\n"
 		"	Display client list in json format\n"
 		"	mac|ip|token is optional, if not specified, all clients are listed\n\n"
@@ -83,7 +156,7 @@ usage(void)
 		"\n"
 		"	customstring is a custom string that will be passed to BinAuth.\n"
 		"\n"
-		"	Example: ndsctl auth 1400 300 1500 500000 1000000 \"This is a Custom String\"\n"
+		"	Example: ndsctl auth 10.13.1.138 1400 300 1500 500000 1000000 \"This is a Custom String\"\n"
 		"\n"
 		"  deauth mac|ip|token\n"
 		"	Deauthenticate user with specified mac, ip or token\n\n"
@@ -100,13 +173,14 @@ usage(void)
 		"  untrust mac\n"
 		"	Untrust the given MAC address\n\n"
 		"  debuglevel n\n"
-		"	Set debug level to n (0=silent, 1=Normal, 2=Info, 3=debug)\n"
+		"	Set debug level to n (0=silent, 1=Normal, 2=Info, 3=debug)\n\n"
+		"  b64decode \"string_to_decode\"\n"
+		"	Base 64 decode the given string\n"
 		"\n"
 	);
 }
 
 static struct argument arguments[] = {
-	{"clients", NULL, NULL},
 	{"json", NULL, NULL},
 	{"status", NULL, NULL},
 	{"stop", NULL, NULL},
@@ -119,6 +193,7 @@ static struct argument arguments[] = {
 	{"unallow", "MAC %s unallowed.\n", "Failed to unallow MAC %s.\n"},
 	{"trust", "MAC %s trusted.\n", "Failed to trust MAC %s.\n"},
 	{"untrust", "MAC %s untrusted.\n", "Failed to untrust MAC %s.\n"},
+	{"b64decode", NULL, NULL},
 	{NULL, NULL, NULL}
 };
 
@@ -184,8 +259,8 @@ static int
 ndsctl_do(const char *socket, const struct argument *arg, const char *param)
 {
 	int sock;
-	char buffer[4096] = {0};
-	char request[1024] = {0};
+	char buffer[MAX_BUF] = {0};
+	char request[MAX_BUF *4 /3] = {0};
 	int len, rlen;
 	int ret;
 
@@ -242,14 +317,61 @@ int
 main(int argc, char **argv)
 {
 	const struct argument* arg;
-	const char *socket;
+	char *socket;
 	int i = 1;
 	int counter;
-	char lockfile[] = "/tmp/ndsctl.lock";
-	char args[512] = {0};
-	char argi[64] = {0};
+	char args[256] = {0};
+	char argi[128] = {0};
+	char str_b64[QUERYMAXLEN] = {0};
+	char mountpoint[128] = {0};
+	char *lockfile;
+	char *cmd;
 	FILE *fd;
 
+	socket = strdup(DEFAULT_SOCKET_FILENAME);
+
+	// check arguments and take action:
+
+	if (argc <= i) {
+		usage();
+		return 1;
+	}
+
+	if (strcmp(argv[1], "-s") == 0) {
+		if (argc >= 2) {
+			socket = strdup(argv[2]);
+			i = 3;
+		} else {
+			usage();
+			return 1;
+		}
+	}
+
+	if (strcmp(argv[i], "b64decode") == 0) {
+		uh_b64decode(str_b64, sizeof(str_b64), argv[i+1], strlen(argv[i+1]));
+		printf("%s", str_b64);
+		return 0;
+	}
+
+	if (strcmp(argv[1], "-h") == 0) {
+		usage();
+		return 0;
+	}
+
+	// Get the tempfs mountpoint
+	safe_asprintf(&cmd, "/usr/lib/opennds/libopennds.sh tmpfs");
+	fd = popen(cmd, "r");
+	if (fd == NULL) {
+		printf("Unable to open library - Terminating");
+		exit(1);
+	}
+	free(cmd);
+	fgets(mountpoint, sizeof(mountpoint), fd);
+	pclose(fd);
+
+	// Create the lock file
+	safe_asprintf(&lockfile, "%s/ndsctl.lock", mountpoint);
+
 	if ((fd = fopen(lockfile, "r")) != NULL) {
 		openlog ("ndsctl", LOG_CONS | LOG_PID | LOG_NDELAY, LOG_LOCAL1);
 		syslog (LOG_NOTICE, "ndsctl is locked by another process");
@@ -262,35 +384,13 @@ main(int argc, char **argv)
 		fd = fopen(lockfile, "w");
 	}
 
-
-	socket = strdup(DEFAULT_SOCK);
-
-	if (argc <= i) {
-		usage();
-		fclose(fd);
-		remove(lockfile);
-		return 0;
-	}
-
-	if (strcmp(argv[1], "-h") == 0) {
-		usage();
-		fclose(fd);
-		remove(lockfile);
-		return 1;
-	}
-
-	if (strcmp(argv[1], "-s") == 0) {
-		if (argc >= 2) {
-			socket = strdup(argv[2]);
-			i = 3;
-		} else {
-			usage();
-			fclose(fd);
-			remove(lockfile);
-			return 1;
-		}
+	// Get the socket path/filename
+	if (strcmp(socket, DEFAULT_SOCKET_FILENAME) == 0) {
+		free(socket);
+		safe_asprintf(&socket, "%s/%s", mountpoint, DEFAULT_SOCKET_FILENAME);
 	}
 
+	// check arguments that need socket access and take action:
 	arg = find_argument(argv[i]);
 
 	if (arg == NULL) {
@@ -304,8 +404,8 @@ main(int argc, char **argv)
 	snprintf(args, sizeof(args), "%s", argv[i+1]);
 
 	if (argc > i) {
-		for (counter=2; counter < argc-1; counter++) {
-			snprintf(argi, sizeof(argi), ",%s", argv[i+counter]);
+		for (counter=i+2; counter < argc; counter++) {
+			snprintf(argi, sizeof(argi), ",%s", argv[counter]);
 			strncat(args, argi, sizeof(args)-1);
 		}
 	}
@@ -313,5 +413,9 @@ main(int argc, char **argv)
 	ndsctl_do(socket, arg, args);
 	fclose(fd);
 	remove(lockfile);
+	free(lockfile);
+	free(socket);
 	return 0;
 }
+
+
