@@ -66,7 +66,6 @@
 #include "debug.h"
 #include "fw_iptables.h"
 
-
 // Defined in main.c
 extern time_t started_time;
 
@@ -91,9 +90,14 @@ int get_client_interface(char* clientif, int clientif_len, const char *climac)
 		debug(LOG_DEBUG, "Client Mac Address: %s", climac);
 		debug(LOG_DEBUG, "Client Connection(s) [localif] [remotemeshnodemac] [localmeshif]: %s", clientif);
 	} else {
-		debug(LOG_ERR, "Failed to get client connections - client probably offline");
-		free (clifcmd);
-		return -1;
+		debug(LOG_INFO, "Failed to get client connections for [%s] - retrying", climac);
+		sleep(1);
+
+		if (execute_ret_url_encoded(clientif, clientif_len - 1, clifcmd) == 0) {
+			debug(LOG_DEBUG, "Client Connection(s) [localif] [remotemeshnodemac] [localmeshif]: %s", clientif);
+		} else {
+			debug(LOG_INFO, "Failed to get client connections for [%s] - giving up", climac);
+		}
 	}
 	free (clifcmd);
 	return 0;
@@ -140,9 +144,15 @@ static int _execute_ret(char* msg, int msg_len, const char *cmd)
 
 	fp = popen(cmd, "r");
 	if (fp == NULL) {
-		debug(LOG_ERR, "popen(): %s", strerror(errno));
-		rc = -1;
-		goto abort;
+		debug(LOG_ERR, "popen(): [%s] Retrying..", strerror(errno));
+		sleep(1);
+		fp = popen(cmd, "r");
+
+		if (fp == NULL) {
+			debug(LOG_ERR, "popen(): [%s] Giving up..", strerror(errno));
+			rc = -1;
+			goto abort;
+		}
 	}
 
 	if (msg && msg_len > 0) {
@@ -227,9 +237,10 @@ get_iface_ip(const char ifname[], int ip6)
 		snprintf(iptype, sizeof(iptype), "inet");
  	}
 
-	snprintf(cmd, sizeof(cmd), "ip address | grep -A4 %s: | grep '%s ' | awk '{print $2}' | awk -F'/' '{printf $1}'",
+	snprintf(cmd, sizeof(cmd), "ip address | grep -A2 ': %s' | grep '%s ' | awk '$NF == \"%s\" {print $2}' | awk -F'/' 'NR==1 {printf $1}'",
 		ifname,
-		iptype
+		iptype,
+		ifname
 	);
 
 	debug(LOG_NOTICE, "Attempting to Bind to interface: %s", ifname);
@@ -411,6 +422,8 @@ ndsctl_status(FILE *fp)
 	t_MAC *allow_mac;
 	t_MAC *block_mac;
 	time_t sysuptime;
+	t_WGP *allowed_wgport;
+	t_WGFQDN *allowed_wgfqdn;
 
 	config = config_get_config();
 
@@ -457,27 +470,32 @@ ndsctl_status(FILE *fp)
 	}
 
 	fprintf(fp, "Client Check Interval: %ds\n", config->checkinterval);
-	fprintf(fp, "Preauth Idle Timeout: %dm\n", config->preauth_idle_timeout);
-	fprintf(fp, "Auth Idle Timeout: %dm\n", config->auth_idle_timeout);
+	fprintf(fp, "Rate Check Window: %d check intervals (%ds)\n", config->rate_check_window, (config->rate_check_window * config->checkinterval));
+	fprintf(fp, "Preauthenticated Client Idle Timeout: %dm\n", config->preauth_idle_timeout);
+	fprintf(fp, "Authenticated Client Idle Timeout: %dm\n", config->auth_idle_timeout);
 
-	if (config->redirectURL) {
-		fprintf(fp, "Redirect URL: %s\n", config->redirectURL);
+	if (config->download_rate > 0) {
+		fprintf(fp, "Download rate limit (default per client): %llu kbit/s\n", config->download_rate);
+	} else {
+		fprintf(fp, "Download rate limit (default per client): no limit\n");
+	}
+	if (config->upload_rate > 0) {
+		fprintf(fp, "Upload rate limit (default per client): %llu kbit/s\n", config->upload_rate);
+	} else {
+		fprintf(fp, "Upload rate limit (default per client): no limit\n");
 	}
 
-	fprintf(fp, "Traffic control: %s\n", config->traffic_control ? "yes" : "no");
-
-	if (config->traffic_control) {
-		if (config->download_rate > 0) {
-			fprintf(fp, "Download rate limit: %llu kbit/s\n", config->download_rate);
-		} else {
-			fprintf(fp, "Download rate limit: none\n");
-		}
-		if (config->upload_rate > 0) {
-			fprintf(fp, "Upload rate limit: %llu kbit/s\n", config->upload_rate);
-		} else {
-			fprintf(fp, "Upload rate limit: none\n");
-		}
+	if (config->download_quota > 0) {
+		fprintf(fp, "Download quota (default per client): %llu kB\n", config->download_quota);
+	} else {
+		fprintf(fp, "Download quota (default per client): no limit\n");
 	}
+	if (config->upload_quota > 0) {
+		fprintf(fp, "Upload quota (default per client): %llu kB\n", config->upload_quota);
+	} else {
+		fprintf(fp, "Upload quota (default per client): no limit\n");
+	}
+
 
 	download_bytes = iptables_fw_total_download();
 	fprintf(fp, "Total download: %llu kByte", download_bytes / 1000);
@@ -528,9 +546,32 @@ ndsctl_status(FILE *fp)
 		}
 
 		fprintf(fp, "  Token: %s\n", client->token ? client->token : "none");
-
 		fprintf(fp, "  State: %s\n", fw_connection_state_as_string(client->fw_connection_state));
 
+		if (client->upload_rate == 0) {
+			fprintf(fp, "  Upload rate limit: not set\n");
+		} else {
+			fprintf(fp, "  Upload rate limit: %llu kb/s\n", client->upload_rate);
+		}
+
+		if (client->download_rate == 0) {
+			fprintf(fp, "  Download rate limit: not set\n");
+		} else {
+			fprintf(fp, "  Download rate limit: %llu kb/s\n", client->download_rate);
+		}
+
+		if (client->upload_quota == 0) {
+			fprintf(fp, "  Upload quota: not set\n");
+		} else {
+			fprintf(fp, "  Upload quota: %llu kB\n", client->upload_quota);
+		}
+
+		if (client->download_quota == 0) {
+			fprintf(fp, "  Download quota: not set\n");
+		} else {
+			fprintf(fp, "  Download quota: %llu kB\n", client->download_quota);
+		}
+
 		download_bytes = client->counters.incoming;
 		upload_bytes = client->counters.outgoing;
 		durationsecs = now - client->session_start;
@@ -540,9 +581,15 @@ ndsctl_status(FILE *fp)
 			durationsecs = 1;
 		}
 
-		fprintf(fp, "  Download: %llu kByte; avg: %.2f kbit/s\n  Upload:   %llu kByte; avg: %.2f kbit/s\n\n",
-				download_bytes / 1000, ((double)download_bytes) / 125 / durationsecs,
-				upload_bytes / 1000, ((double)upload_bytes) / 125 / durationsecs);
+		fprintf(fp, "  Upload this session: %llu kB; Session avg: %.2f kb/s\n",
+			upload_bytes / 1000,
+			((double)upload_bytes) / 125 / durationsecs)
+		;
+
+		fprintf(fp, "  Download this session: %llu kB; Session avg: %.2f kb/s\n\n",
+			download_bytes / 1000,
+			((double)download_bytes) / 125 / durationsecs)
+		;
 
 		indx++;
 		client = client->next;
@@ -589,96 +636,118 @@ ndsctl_status(FILE *fp)
 		fprintf(fp, " none\n");
 	}
 
+	fprintf(fp, "Walled Garden FQDNs:");
+
+	if (config->walledgarden_fqdn_list != NULL) {
+		fprintf(fp, "\n");
+		for (allowed_wgfqdn = config->walledgarden_fqdn_list; allowed_wgfqdn != NULL; allowed_wgfqdn = allowed_wgfqdn->next) {
+			fprintf(fp, "  %s\n", allowed_wgfqdn->wgfqdn);
+		}
+	} else {
+		fprintf(fp, " none\n");
+	}
+
+	fprintf(fp, "Walled Garden Ports:");
+
+	if (config->walledgarden_port_list != NULL) {
+		fprintf(fp, "\n");
+		for (allowed_wgport = config->walledgarden_port_list; allowed_wgport != NULL; allowed_wgport = allowed_wgport->next) {
+			fprintf(fp, "  %u\n", allowed_wgport->wgport);
+		}
+	} else {
+		fprintf(fp, " none\n");
+	}
+
 	fprintf(fp, "========\n");
 }
 
-void
-ndsctl_clients(FILE *fp)
-{
-	t_client *client;
-	int indx;
-	unsigned long int now, durationsecs = 0;
-	unsigned long long int download_bytes, upload_bytes;
-
-	now = time(NULL);
-
-	// Update the client's counters so info is current
-	iptables_fw_counters_update();
-
-	LOCK_CLIENT_LIST();
-
-	fprintf(fp, "%d\n", get_client_list_length());
-
-	client = client_get_first_client();
-	if (client) {
-		fprintf(fp, "\n");
-	}
-
-	indx = 0;
-	while (client != NULL) {
-		fprintf(fp, "client_id=%d\n", indx);
-		fprintf(fp, "ip=%s\nmac=%s\n", client->ip, client->mac);
-		fprintf(fp, "added=%lld\n", (long long) client->session_start);
-		fprintf(fp, "active=%lld\n", (long long) client->counters.last_updated);
-		if (client->session_start) {
-			fprintf(fp, "duration=%lu\n", now - client->session_start);
-		} else {
-			fprintf(fp, "duration=%lu\n", 0ul);
-		}
-		fprintf(fp, "token=%s\n", client->token ? client->token : "none");
-		fprintf(fp, "state=%s\n", fw_connection_state_as_string(client->fw_connection_state));
-
-		durationsecs = now - client->session_start;
-		download_bytes = client->counters.incoming;
-		upload_bytes = client->counters.outgoing;
-
-		fprintf(fp, "downloaded=%llu\n", download_bytes/1000);
-		fprintf(fp, "avg_down_speed=%.2f\n", ((double)download_bytes) / 125 / durationsecs);
-		fprintf(fp, "uploaded=%llu\n", upload_bytes/1000);
-		fprintf(fp, "avg_up_speed=%.2f\n\n", ((double)upload_bytes) / 125 / durationsecs);
-
-		indx++;
-		client = client->next;
-	}
-
-	UNLOCK_CLIENT_LIST();
-}
-
 static void
-ndsctl_json_client(FILE *fp, const t_client *client, time_t now)
+ndsctl_json_client(FILE *fp, const t_client *client, time_t now, char *indent)
 {
 	unsigned long int durationsecs;
 	unsigned long long int download_bytes, upload_bytes;
+	char clientif[64] = {0};
+	s_config *config;
 
-	fprintf(fp, "\"id\":\"%d\",\n", client->id);
-	fprintf(fp, "\"ip\":\"%s\",\n", client->ip);
-	fprintf(fp, "\"mac\":\"%s\",\n", client->mac);
-	fprintf(fp, "\"added\":\"%lld\",\n", (long long) client->session_start);
-	fprintf(fp, "\"active\":\"%lld\",\n", (long long) client->counters.last_updated);
-	if (client->session_start) {
-		fprintf(fp, "\"duration\":\"%lu\",\n", now - client->session_start);
+	config = config_get_config();
+	get_client_interface(clientif, sizeof(clientif), client->mac);
+
+	fprintf(fp, "  %s\"gatewayname\":\"%s\",\n", indent, config->url_encoded_gw_name);
+	fprintf(fp, "  %s\"version\":\"%s\",\n", indent, VERSION);
+	fprintf(fp, "  %s\"mac\":\"%s\",\n", indent, client->mac);
+	fprintf(fp, "  %s\"ip\":\"%s\",\n", indent, client->ip);
+	fprintf(fp, "  %s\"clientif\":\"%s\",\n", indent, clientif);
+	fprintf(fp, "  %s\"session_start\":\"%lld\",\n", indent, (long long) client->session_start);
+
+	if (client->session_end == 0) {
+		fprintf(fp, "  %s\"session_end\":\"null\",\n", indent);
 	} else {
-		fprintf(fp, "\"duration\":\"%lu\",\n", 0ul);
+		fprintf(fp, "  %s\"session_end\":\"%lld\",\n", indent, (long long) client->session_end);
 	}
-	fprintf(fp, "\"token\":\"%s\",\n", client->token ? client->token : "none");
-	fprintf(fp, "\"state\":\"%s\",\n", fw_connection_state_as_string(client->fw_connection_state));
+
+	fprintf(fp, "  %s\"last_active\":\"%lld\",\n", indent, (long long) client->counters.last_updated);
+	fprintf(fp, "  %s\"token\":\"%s\",\n", indent, client->token ? client->token : "none");
+	fprintf(fp, "  %s\"state\":\"%s\",\n", indent, fw_connection_state_as_string(client->fw_connection_state));
 
 	durationsecs = now - client->session_start;
 	download_bytes = client->counters.incoming;
 	upload_bytes = client->counters.outgoing;
 
-	fprintf(fp, "\"downloaded\":\"%llu\",\n", download_bytes / 1000);
-	fprintf(fp, "\"avg_down_speed\":\"%.2f\",\n", ((double)download_bytes) / 125 / durationsecs);
-	fprintf(fp, "\"uploaded\":\"%llu\",\n", upload_bytes / 1000);
-	fprintf(fp, "\"avg_up_speed\":\"%.2f\"\n", ((double)upload_bytes)/ 125 / durationsecs);
+	if (client->upload_rate == 0) {
+		fprintf(fp, "  %s\"upload_rate_limit\":\"null\",\n", indent);
+	} else {
+		fprintf(fp, "  %s\"upload_rate_limit\":\"%llu\",\n", indent, client->upload_rate);
+	}
+
+	if (client->download_rate == 0) {
+		fprintf(fp, "  %s\"download_rate_limit\":\"null\",\n", indent);
+	} else {
+		fprintf(fp, "  %s\"download_rate_limit\":\"%llu\",\n", indent, client->download_rate);
+	}
+
+	if (client->upload_quota == 0) {
+		fprintf(fp, "  %s\"upload_quota\":\"null\",\n", indent);
+	} else {
+		fprintf(fp, "  %s\"upload_quota\":\"%llu\",\n", indent, client->upload_quota);
+	}
+
+	if (client->download_quota == 0) {
+		fprintf(fp, "  %s\"download_quota\":\"null\",\n", indent);
+	} else {
+		fprintf(fp, "  %s\"download_quota\":\"%llu\",\n", indent, client->download_quota);
+	}
+
+	// prevent divison by 0
+	if (durationsecs < 1) {
+		durationsecs = 1;
+	}
+
+	fprintf(fp, "  %s\"upload_this_session\":\"%llu\",\n",
+		indent,
+		(upload_bytes / 1000)
+	);
+
+	fprintf(fp, "  %s\"upload_session_avg\":\"%.2f\",\n",
+		indent,
+		(double)upload_bytes / 125 / durationsecs
+	);
+
+	fprintf(fp, "  %s\"download_this_session\":\"%llu\",\n",
+		indent,
+		(download_bytes / 1000)
+	);
+
+	fprintf(fp, "  %s\"download_session_avg\":\"%.2f\"\n",
+		indent,
+		(double)download_bytes / 125 / durationsecs
+	);
 }
 
 static void
-ndsctl_json_one(FILE *fp, const char *arg)
+ndsctl_json_one(FILE *fp, const char *arg, char *indent)
 {
 	t_client *client;
 	time_t now;
-
 	now = time(NULL);
 
 	// Update the client's counters so info is current
@@ -690,7 +759,7 @@ ndsctl_json_one(FILE *fp, const char *arg)
 
 	if (client) {
 		fprintf(fp, "{\n");
-		ndsctl_json_client(fp, client, now);
+		ndsctl_json_client(fp, client, now, indent);
 		fprintf(fp, "}\n");
 	} else {
 		fprintf(fp, "{}\n");
@@ -700,7 +769,7 @@ ndsctl_json_one(FILE *fp, const char *arg)
 }
 
 static void
-ndsctl_json_all(FILE *fp)
+ndsctl_json_all(FILE *fp, char *indent)
 {
 	t_client *client;
 	time_t now;
@@ -717,63 +786,67 @@ ndsctl_json_all(FILE *fp)
 
 	LOCK_CLIENT_LIST();
 
-	fprintf(fp, "{\n\"client_list_length\": \"%d\",\n", get_client_list_length());
+	fprintf(fp, "{\n  \"client_list_length\":\"%d\",\n", get_client_list_length());
 
 	client = client_get_first_client();
 
-	fprintf(fp, "\"clients\":{\n");
+	fprintf(fp, "  \"clients\":{\n");
 
 	while (client != NULL) {
-		fprintf(fp, "\"%s\":{\n", client->mac);
-		ndsctl_json_client(fp, client, now);
+		fprintf(fp, "%s\"%s\":{\n", indent, client->mac);
+		ndsctl_json_client(fp, client, now, indent);
 
 		client = client->next;
 		if (client) {
-			fprintf(fp, "},\n");
+			fprintf(fp, "%s},\n", indent);
 		} else {
-			fprintf(fp, "}\n");
+			fprintf(fp, "%s}\n", indent);
 		}
 	}
 
-	fprintf(fp, "}\n}\n");
-
 	UNLOCK_CLIENT_LIST();
 
 	// Trusted mac list
 	if (config->trustedmaclist != NULL) {
-
+		fprintf(fp, "  },\n");
 		// count the number of trusted mac addresses
 		for (trust_mac = config->trustedmaclist; trust_mac != NULL; trust_mac = trust_mac->next) {
 			count++;
 		}
 
 		// output the count of trusted macs and list them in json array format
-		fprintf(fp, "{\n\"trusted_list_length\": \"%d\",\n", count);
-		fprintf(fp, "\"trusted\":[\n");
+		fprintf(fp, "  \"trusted_list_length\":\"%d\",\n", count);
+		fprintf(fp, "  \"trusted\":[\n");
 
 		for (trust_mac = config->trustedmaclist; trust_mac != NULL; trust_mac = trust_mac->next) {
 
 			if (count > 1) {
-				fprintf(fp, "\"%s\",\n", trust_mac->mac);
+				fprintf(fp, "    \"%s\",\n", trust_mac->mac);
 				count--;
 			} else {
-				fprintf(fp, "\"%s\"\n", trust_mac->mac);
+				fprintf(fp, "    \"%s\"\n", trust_mac->mac);
 			}
 		}
 
-		fprintf(fp, "]\n}\n");
+		fprintf(fp, "  ]\n");
+	} else {
+		fprintf(fp, "  }\n");
 	}
+	fprintf(fp, "}\n");
 }
 
 void
 ndsctl_json(FILE *fp, const char *arg)
 {
+	char indent[5] = {0};
 	//if (arg && strlen(arg)) {
 	debug(LOG_DEBUG, "arg [%s %d]", arg, strlen(arg));
 	if (strlen(arg) > 6) {
-		ndsctl_json_one(fp, arg);
+		//snprintf(indent, sizeof(indent), "%s", "");
+		ndsctl_json_one(fp, arg, indent);
 	} else {
-		ndsctl_json_all(fp);
+		snprintf(indent, sizeof(indent), "%s", "    ");
+		ndsctl_json_all(fp, indent);
 	}
 }
 
